@@ -27,6 +27,10 @@ PYTHONBIN=$1
 CONFIG="$2"
 PYVERSION=$($PYTHONBIN -c "import sys; print(sys.implementation.name + sys.version.split()[0])")
 
+# This is needed because of scikit, otherwise it will include files from another
+# python version installed in the system. 
+PYINCLUDE=$($PYTHONBIN -c "import sysconfig; print(sysconfig.get_paths()['include'])")
+
 # setup and activate venv
 echo "Making venv with $PYTHONBIN"
 ENVDIR=env$PYVERSION
@@ -35,7 +39,7 @@ source ./$ENVDIR/bin/activate
 
 # install packages
 pip install -q --upgrade pip setuptools auditwheel
-pip install -q Cython pytest toml scikit-build flex pyparsing 
+pip install -q Cython pytest tomli scikit-build flex pyparsing 
 if [ "$(uname)" == "Darwin" ]; then
     # Mac version of auditwheel
     pip install -q delocate
@@ -44,7 +48,11 @@ fi
 # configure cvc5
 echo "Configuring"
 rm -rf build_wheel/
-python contrib/packaging_python/mk_build_dir.py $CONFIG --python-bindings --name=build_wheel
+
+# This new command like is supposed to ensure that we use the python 
+# version from the virtual environment that is activated. 
+# Once we remove scikit, this will not be necessary.
+./configure.sh $CONFIG --python-bindings --name=build_wheel -DPython_FIND_VIRTUALENV=ONLY -DPYTHON_LIBRARY=$VIRTUAL_ENV/lib -DPYTHON_INCLUDE_DIR=$PYINCLUDE
 
 # building wheel
 echo "Building pycvc5 wheel"
