@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -148,21 +148,38 @@ Node getNullTerminator(Kind k, TypeNode tn)
       nullTerm = nm->mkNode(REGEXP_ALL);
       break;
     case BITVECTOR_AND:
-      nullTerm = theory::bv::utils::mkOnes(tn.getBitVectorSize());
+      // it may be the case that we are an abstract type, which we guard here
+      // and return the null node.
+      if (tn.isBitVector())
+      {
+        nullTerm = theory::bv::utils::mkOnes(tn.getBitVectorSize());
+      }
       break;
     case BITVECTOR_OR:
     case BITVECTOR_ADD:
     case BITVECTOR_XOR:
-      nullTerm = theory::bv::utils::mkZero(tn.getBitVectorSize());
+      if (tn.isBitVector())
+      {
+        nullTerm = theory::bv::utils::mkZero(tn.getBitVectorSize());
+      }
       break;
     case BITVECTOR_MULT:
-      nullTerm = theory::bv::utils::mkOne(tn.getBitVectorSize());
+      if (tn.isBitVector())
+      {
+        nullTerm = theory::bv::utils::mkOne(tn.getBitVectorSize());
+      }
       break;
     case FINITE_FIELD_ADD:
-      nullTerm = nm->mkConst(FiniteFieldValue(Integer(0), tn.getFfSize()));
+      if (tn.isFiniteField())
+      {
+        nullTerm = nm->mkConst(FiniteFieldValue(Integer(0), tn.getFfSize()));
+      }
       break;
     case FINITE_FIELD_MULT:
-      nullTerm = nm->mkConst(FiniteFieldValue(Integer(1), tn.getFfSize()));
+      if (tn.isFiniteField())
+      {
+        nullTerm = nm->mkConst(FiniteFieldValue(Integer(1), tn.getFfSize()));
+      }
       break;
     default:
       // not handled as null-terminated
@@ -244,11 +261,20 @@ Node narySubstitute(Node src,
         {
           // n-ary operators cannot be parameterized
           Assert(cur.getMetaKind() != metakind::PARAMETERIZED);
-          ret = children.empty()
-                    ? getNullTerminator(cur.getKind(), cur.getType())
-                    : (children.size() == 1
-                           ? children[0]
-                           : nm->mkNode(cur.getKind(), children));
+          if (children.empty())
+          {
+            ret = getNullTerminator(cur.getKind(), cur.getType());
+            // if we don't know the null terminator, just return null now
+            if (ret.isNull())
+            {
+              return ret;
+            }
+          }
+          else
+          {
+            ret = (children.size() == 1 ? children[0]
+                                        : nm->mkNode(cur.getKind(), children));
+          }
         }
         else
         {
