@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Andrew Reynolds, Andres Noetzli, Yoni Zohar
+ *   Andrew Reynolds, Andres Noetzli, Abdalrhman Mohamed
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -244,6 +244,13 @@ bool isBooleanConnective(TNode cur);
 /** Is n a theory atom? */
 bool isTheoryAtom(TNode cur);
 
+/**
+ * Returns true iff the node n contains a subterm with abstract type.
+ * @param n The node under investigation
+ * @return true iff this node contains a subterm with abstract type
+ */
+bool hasAbstractSubterm(TNode n);
+
 }  // namespace expr
 }  // namespace cvc5::internal
 
