@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -709,13 +709,15 @@ class NodeManager
   TypeNode mkSort();
 
   /** Make a new sort with the given name of arity 0. */
-  TypeNode mkSort(const std::string& name);
+  TypeNode mkSort(const std::string& name, bool fresh = true);
 
   /** Make a new sort by parameterizing the given sort constructor. */
   TypeNode mkSort(TypeNode constructor, const std::vector<TypeNode>& children);
 
   /** Make a new sort with the given name and arity. */
-  TypeNode mkSortConstructor(const std::string& name, size_t arity);
+  TypeNode mkSortConstructor(const std::string& name,
+                             size_t arity,
+                             bool fresh = true);
 
   /** Make an unresolved datatype sort */
   TypeNode mkUnresolvedDatatypeSort(const std::string& name, size_t arity = 0);
@@ -952,11 +954,14 @@ class NodeManager
    * submodule is the interface for constructing internal variables
    * (see expr/skolem_manager.h).
    */
-  Node mkVar(const std::string& name, const TypeNode& type);
+  Node mkVar(const std::string& name, const TypeNode& type, bool fresh = true);
 
   /** Create a variable with the given type. */
   Node mkVar(const TypeNode& type);
 
+  /** Make a new sort with the given name and arity. */
+  TypeNode mkSortConstructorInternal(const std::string& name, size_t arity);
+
   /** The skolem manager */
   std::unique_ptr<SkolemManager> d_skManager;
   /** The bound variable manager */
@@ -1019,6 +1024,9 @@ class NodeManager
   /** A list of oracles owned by this node manager */
   std::vector<std::unique_ptr<Oracle>> d_oracles;
 
+  /** A mapping for sorts allocated by mkSortConstructor where fresh is false */
+  std::map<std::pair<std::string, size_t>, TypeNode> d_nfreshSorts;
+
   TupleTypeCache d_tt_cache;
   RecTypeCache d_rt_cache;
 }; /* class NodeManager */
