@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Morgan Deters, Aina Niemetz, Mathias Preiner
+ *   Morgan Deters, Andrew Reynolds, Aina Niemetz
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -38,7 +38,6 @@ TypeNode TypeChecker::preComputeType(NodeManager* nodeManager, TNode n)
     case kind::SKOLEM:
     case kind::BOUND_VARIABLE:
     case kind::INST_CONSTANT:
-    case kind::BOOLEAN_TERM_VARIABLE:
     case kind::RAW_SYMBOL:
       // variable kinds have their type marked as an attribute upon construction
       typeNode = nodeManager->getAttribute(n, TypeAttr());
@@ -47,7 +46,9 @@ TypeNode TypeChecker::preComputeType(NodeManager* nodeManager, TNode n)
       typeNode = nodeManager->builtinOperatorType();
       break;
 
-      // !!! will auto-generate preComputeType rules when they are available
+      // clang-format off
+${pretyperules}
+      // clang-format on
 
     default:
       // not handled
@@ -66,13 +67,6 @@ TypeNode TypeChecker::computeType(NodeManager* nodeManager,
   // Infer the type
   switch (n.getKind())
   {
-    case kind::VARIABLE:
-    case kind::SKOLEM:
-      typeNode = nodeManager->getAttribute(n, TypeAttr());
-      break;
-    case kind::BUILTIN:
-      typeNode = nodeManager->builtinOperatorType();
-      break;
 
       // clang-format off
 ${typerules}
@@ -83,10 +77,6 @@ ${typerules}
       Unhandled() << " " << n.getKind();
   }
 
-  nodeManager->setAttribute(n, TypeAttr(), typeNode);
-  nodeManager->setAttribute(n, TypeCheckedAttr(),
-                            check || nodeManager->getAttribute(n, TypeCheckedAttr()));
-
   return typeNode;
 
 } /* TypeChecker::computeType */
