@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Gereon Kremer, Andrew Reynolds, Morgan Deters
+ *   Andrew Reynolds, Gereon Kremer, Morgan Deters
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -19,16 +19,18 @@
 #  include <sys/resource.h>
 #endif /* ! __WIN32__ */
 
+#include <cvc5/cvc5_parser.h>
+
 #include <iomanip>
 #include <iostream>
 #include <memory>
 #include <string>
 #include <vector>
 
-#include "main/main.h"
-#include "parser/api/cpp/command.h"
-#include "smt/solver_engine.h"
 #include "base/output.h"
+#include "main/main.h"
+#include "parser/commands.h"
+#include "smt/solver_engine.h"
 
 using namespace cvc5::parser;
 
@@ -95,19 +97,21 @@ bool CommandExecutor::doCommand(Command* cmd)
 {
   // formerly was guarded by verbosity > 2
   Trace("cmd-exec") << "Invoking: " << *cmd << std::endl;
-  return doCommandSingleton(cmd);
+  return doCommandSingleton(cmd->toCmd());
 }
 
 void CommandExecutor::reset()
 {
   printStatistics(d_solver->getDriverOptions().err());
-  Command::resetSolver(d_solver.get());
+  Cmd::resetSolver(d_solver.get());
 }
 
-bool CommandExecutor::doCommandSingleton(Command* cmd)
+bool CommandExecutor::doCommandSingleton(Cmd* cmd)
 {
-  bool status = solverInvoke(
-      d_solver.get(), d_symman.get(), cmd, d_solver->getDriverOptions().out());
+  bool status = solverInvoke(d_solver.get(),
+                             d_symman->toSymManager(),
+                             cmd,
+                             d_solver->getDriverOptions().out());
 
   cvc5::Result res;
   bool hasResult = false;
@@ -135,7 +139,7 @@ bool CommandExecutor::doCommandSingleton(Command* cmd)
   if (status) {
     bool isResultUnsat = res.isUnsat();
     bool isResultSat = res.isSat();
-    std::vector<std::unique_ptr<Command> > getterCommands;
+    std::vector<std::unique_ptr<Cmd> > getterCommands;
     if (d_solver->getOptionInfo("dump-models").boolValue()
         && (isResultSat
             || (res.isUnknown()
@@ -187,14 +191,14 @@ bool CommandExecutor::doCommandSingleton(Command* cmd)
 }
 
 bool CommandExecutor::solverInvoke(cvc5::Solver* solver,
-                                   SymbolManager* sm,
-                                   Command* cmd,
+                                   SymManager* sm,
+                                   Cmd* cmd,
                                    std::ostream& out)
 {
   // print output for -o raw-benchmark
   if (solver->isOutputOn("raw-benchmark"))
   {
-    cmd->toStream(solver->getOutput("raw-benchmark"));
+    solver->getOutput("raw-benchmark") << cmd->toString();
   }
 
   // In parse-only mode, we do not invoke any of the commands except define-*
@@ -202,7 +206,8 @@ bool CommandExecutor::solverInvoke(cvc5::Solver* solver,
   // commands because they add function names to the symbol table.
   if (d_parseOnly && dynamic_cast<SetBenchmarkLogicCommand*>(cmd) == nullptr
       && dynamic_cast<ResetCommand*>(cmd) == nullptr
-      && dynamic_cast<DeclarationDefinitionCommand*>(cmd) == nullptr)
+      && dynamic_cast<DeclarationDefinitionCommand*>(cmd) == nullptr
+      && dynamic_cast<DatatypeDeclarationCommand*>(cmd) == nullptr)
   {
     return true;
   }
