@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Morgan Deters, Christopher L. Conway, Gereon Kremer
+ *   Morgan Deters, Andrew Reynolds, Christopher L. Conway
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -40,13 +40,14 @@
 #endif   /* HAVE_LIBEDITLINE */
 
 #include <cvc5/cvc5.h>
+#include <cvc5/cvc5_parser.h>
 
 #include "base/check.h"
 #include "base/output.h"
 #include "main/command_executor.h"
-#include "parser/api/cpp/command.h"
-#include "parser/api/cpp/input_parser.h"
-#include "parser/api/cpp/symbol_manager.h"
+#include "parser/commands.h"
+#include "parser/parser_exception.h"
+#include "parser/sym_manager.h"
 #include "theory/logic_info.h"
 
 using namespace std;
@@ -71,10 +72,6 @@ static const std::string smt2_commands[] = {
 #include "main/smt2_tokens.h"
 };/* smt2_commands */
 
-static const std::string tptp_commands[] = {
-#include "main/tptp_tokens.h"
-};/* tptp_commands */
-
 static const std::string* commandsBegin;
 static const std::string* commandsEnd;
 
@@ -88,19 +85,15 @@ InteractiveShell::InteractiveShell(main::CommandExecutor* cexec,
                                    bool isInteractive)
     : d_cexec(cexec),
       d_solver(cexec->getSolver()),
-      d_symman(cexec->getSymbolManager()),
+      d_symman(cexec->getSymbolManager()->toSymManager()),
       d_in(in),
       d_out(out),
       d_isInteractive(isInteractive),
       d_quit(false)
 {
-  if (d_solver->getOptionInfo("force-logic").setByUser)
-  {
-    LogicInfo tmp(d_solver->getOption("force-logic"));
-    d_symman->forceLogic(tmp.getLogicString());
-  }
   /* Create parser with bogus input. */
-  d_parser.reset(new cvc5::parser::InputParser(d_solver, d_symman));
+  d_parser.reset(
+      new cvc5::parser::InputParser(d_solver, cexec->getSymbolManager()));
   // initialize for incremental string input
   d_parser->setIncrementalStringInput(d_solver->getOption("input-language"),
                                       INPUT_FILENAME);
@@ -116,14 +109,7 @@ InteractiveShell::InteractiveShell(main::CommandExecutor* cexec,
     ::using_history();
 
     std::string lang = d_solver->getOption("input-language");
-    if (lang == "LANG_TPTP")
-    {
-      d_historyFilename = string(getenv("HOME")) + "/.cvc5_history_tptp";
-      commandsBegin = tptp_commands;
-      commandsEnd =
-          tptp_commands + sizeof(tptp_commands) / sizeof(*tptp_commands);
-    }
-    else if (lang == "LANG_SMTLIB_V2_6")
+    if (lang == "LANG_SMTLIB_V2_6")
     {
       d_historyFilename = string(getenv("HOME")) + "/.cvc5_history_smtlib2";
       commandsBegin = smt2_commands;
@@ -157,7 +143,7 @@ InteractiveShell::InteractiveShell(main::CommandExecutor* cexec,
 #else  /* HAVE_LIBEDITLINE */
   d_usingEditline = false;
 #endif /* HAVE_LIBEDITLINE */
-}/* InteractiveShell::InteractiveShell() */
+} /* InteractiveShell::InteractiveShell() */
 
 InteractiveShell::~InteractiveShell() {
 #if HAVE_LIBEDITLINE
