@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Abdalrhman Mohamed, Andrew Reynolds, Gereon Kremer
+ *   Abdalrhman Mohamed, Andrew Reynolds, Andres Noetzli
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -17,13 +17,14 @@
 #include <sstream>
 #include <string>
 
+#include "expr/node.h"
 #include "options/base_options.h"
 #include "options/language.h"
 #include "options/printer_options.h"
 #include "printer/ast/ast_printer.h"
 #include "printer/smt2/smt2_printer.h"
-#include "printer/tptp/tptp_printer.h"
 #include "proof/unsat_core.h"
+#include "smt/model.h"
 #include "theory/quantifiers/instantiation_list.h"
 
 using namespace std;
@@ -40,9 +41,6 @@ unique_ptr<Printer> Printer::makePrinter(Language lang)
       return unique_ptr<Printer>(
           new printer::smt2::Smt2Printer(printer::smt2::smt2_6_variant));
 
-    case Language::LANG_TPTP:
-      return unique_ptr<Printer>(new printer::tptp::TptpPrinter());
-
     case Language::LANG_SYGUS_V2:
       // sygus version 2.0 does not have discrepancies with smt2, hence we use
       // a normal smt2 variant here.
@@ -209,6 +207,7 @@ void Printer::toStreamCmdPop(std::ostream& out, uint32_t nscopes) const
 
 void Printer::toStreamCmdDeclareFunction(std::ostream& out,
                                          const std::string& id,
+                                         const std::vector<TypeNode>& argTypes,
                                          TypeNode type) const
 {
   printUnknownCommand(out, "declare-fun");
@@ -221,7 +220,14 @@ void Printer::toStreamCmdDeclareFunction(std::ostream& out, const Node& v) const
   // assigned names.
   std::stringstream ss;
   toStream(ss, v);
-  toStreamCmdDeclareFunction(out, ss.str(), v.getType());
+  TypeNode vt = v.getType();
+  std::vector<TypeNode> argTypes;
+  if (vt.isFunction())
+  {
+    argTypes = vt.getArgTypes();
+    vt = vt.getRangeType();
+  }
+  toStreamCmdDeclareFunction(out, ss.str(), argTypes, vt);
 }
 
 void Printer::toStreamCmdDeclarePool(std::ostream& out,
@@ -234,6 +240,7 @@ void Printer::toStreamCmdDeclarePool(std::ostream& out,
 
 void Printer::toStreamCmdDeclareOracleFun(std::ostream& out,
                                           const std::string& id,
+                                          const std::vector<TypeNode>& argTypes,
                                           TypeNode type,
                                           const std::string& binName) const
 {
@@ -241,11 +248,21 @@ void Printer::toStreamCmdDeclareOracleFun(std::ostream& out,
 }
 
 void Printer::toStreamCmdDeclareType(std::ostream& out,
-                                     TypeNode type) const
+                                     const std::string& id,
+                                     size_t arity) const
 {
   printUnknownCommand(out, "declare-sort");
 }
 
+void Printer::toStreamCmdDeclareType(std::ostream& out, TypeNode type) const
+{
+  Assert(type.isUninterpretedSort() || type.isUninterpretedSortConstructor());
+  size_t arity = type.isUninterpretedSortConstructor()
+                     ? type.getUninterpretedSortConstructorArity()
+                     : 0;
+  toStreamCmdDeclareType(out, type.getName(), arity);
+}
+
 void Printer::toStreamCmdDefineType(std::ostream& out,
                                     const std::string& id,
                                     const std::vector<TypeNode>& params,
@@ -341,19 +358,19 @@ void Printer::toStreamCmdQuery(std::ostream& out, Node n) const
 }
 
 void Printer::toStreamCmdDeclareVar(std::ostream& out,
-                                    Node var,
+                                    const std::string& id,
                                     TypeNode type) const
 {
   printUnknownCommand(out, "declare-var");
 }
 
 void Printer::toStreamCmdSynthFun(std::ostream& out,
-                                  Node f,
+                                  const std::string& id,
                                   const std::vector<Node>& vars,
-                                  bool isInv,
+                                  TypeNode rangeType,
                                   TypeNode sygusType) const
 {
-  printUnknownCommand(out, isInv ? "synth-inv" : "synth-fun");
+  printUnknownCommand(out, "synth-fun");
 }
 
 void Printer::toStreamCmdConstraint(std::ostream& out, Node n) const
@@ -381,6 +398,18 @@ void Printer::toStreamCmdCheckSynthNext(std::ostream& out) const
   printUnknownCommand(out, "check-synth-next");
 }
 
+void Printer::toStreamCmdFindSynth(std::ostream& out,
+                                   modes::FindSynthTarget fst,
+                                   TypeNode sygusType) const
+{
+  printUnknownCommand(out, "find-synth");
+}
+
+void Printer::toStreamCmdFindSynthNext(std::ostream& out) const
+{
+  printUnknownCommand(out, "find-synth-next");
+}
+
 void Printer::toStreamCmdSimplify(std::ostream& out, Node n) const
 {
   printUnknownCommand(out, "simplify");
@@ -473,6 +502,11 @@ void Printer::toStreamCmdGetDifficulty(std::ostream& out) const
   printUnknownCommand(out, "get-difficulty");
 }
 
+void Printer::toStreamCmdGetTimeoutCore(std::ostream& out) const
+{
+  printUnknownCommand(out, "get-timeout-core");
+}
+
 void Printer::toStreamCmdGetLearnedLiterals(std::ostream& out,
                                             modes::LearnedLitType t) const
 {
