@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Abdalrhman Mohamed, Andrew Reynolds, Aina Niemetz
+ *   Abdalrhman Mohamed, Andrew Reynolds, Andres Noetzli
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -18,19 +18,31 @@
 #ifndef CVC5__PRINTER__PRINTER_H
 #define CVC5__PRINTER__PRINTER_H
 
-#include <string>
+#include <cvc5/cvc5_export.h>
 
-#include "expr/node.h"
+#include <memory>
+#include <string>
+#include <vector>
+
+#include "expr/kind.h"
 #include "options/language.h"
-#include "smt/model.h"
 #include "util/result.h"
 
 namespace cvc5::internal {
 
+template <bool ref_count>
+class NodeTemplate;
+typedef NodeTemplate<true> Node;
+typedef NodeTemplate<false> TNode;
+class TypeNode;
 class UnsatCore;
 struct InstantiationList;
 struct SkolemList;
 
+namespace smt {
+class Model;
+}
+
 class CVC5_EXPORT Printer
 {
  public:
@@ -101,6 +113,7 @@ class CVC5_EXPORT Printer
   /** Print declare-fun command */
   virtual void toStreamCmdDeclareFunction(std::ostream& out,
                                           const std::string& id,
+                                          const std::vector<TypeNode>& argTypes,
                                           TypeNode type) const;
   /** Variant of above for a pre-existing variable */
   void toStreamCmdDeclareFunction(std::ostream& out, const Node& v) const;
@@ -110,14 +123,19 @@ class CVC5_EXPORT Printer
                                       TypeNode type,
                                       const std::vector<Node>& initValue) const;
   /** Print declare-oracle-fun command */
-  virtual void toStreamCmdDeclareOracleFun(std::ostream& out,
-                                           const std::string& id,
-                                           TypeNode type,
-                                           const std::string& binName) const;
+  virtual void toStreamCmdDeclareOracleFun(
+      std::ostream& out,
+      const std::string& id,
+      const std::vector<TypeNode>& argTypes,
+      TypeNode type,
+      const std::string& binName) const;
 
   /** Print declare-sort command */
   virtual void toStreamCmdDeclareType(std::ostream& out,
-                                      TypeNode type) const;
+                                      const std::string& id,
+                                      size_t arity) const;
+  /** Variant of above that takes the type */
+  void toStreamCmdDeclareType(std::ostream& out, TypeNode type) const;
 
   /** Print define-sort command */
   virtual void toStreamCmdDefineType(std::ostream& out,
@@ -162,15 +180,15 @@ class CVC5_EXPORT Printer
 
   /** Print declare-var command */
   virtual void toStreamCmdDeclareVar(std::ostream& out,
-                                     Node var,
+                                     const std::string& id,
                                      TypeNode type) const;
 
   /** Print synth-fun command */
   virtual void toStreamCmdSynthFun(std::ostream& out,
-                                   Node f,
+                                   const std::string& id,
                                    const std::vector<Node>& vars,
-                                   bool isInv,
-                                   TypeNode sygusType = TypeNode::null()) const;
+                                   TypeNode rangeType,
+                                   TypeNode sygusType) const;
 
   /** Print constraint command */
   virtual void toStreamCmdConstraint(std::ostream& out, Node n) const;
@@ -188,6 +206,14 @@ class CVC5_EXPORT Printer
   /** Print check-synth-next command */
   virtual void toStreamCmdCheckSynthNext(std::ostream& out) const;
 
+  /** Print find-synth command */
+  virtual void toStreamCmdFindSynth(std::ostream& out,
+                                    modes::FindSynthTarget fst,
+                                    TypeNode sygusType) const;
+
+  /** Print find-synth-next command */
+  virtual void toStreamCmdFindSynthNext(std::ostream& out) const;
+
   /** Print simplify command */
   virtual void toStreamCmdSimplify(std::ostream& out, Node n) const;
 
@@ -248,6 +274,9 @@ class CVC5_EXPORT Printer
   /** Print get-difficulty command */
   virtual void toStreamCmdGetDifficulty(std::ostream& out) const;
 
+  /** Print get-timeout-core command */
+  virtual void toStreamCmdGetTimeoutCore(std::ostream& out) const;
+
   /** Print get-learned-literals command */
   virtual void toStreamCmdGetLearnedLiterals(std::ostream& out,
                                              modes::LearnedLitType t) const;
