@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -38,8 +38,10 @@
 #include "options/language.h"
 #include "printer/let_binding.h"
 #include "proof/unsat_core.h"
+#include "smt/model.h"
 #include "theory/arrays/theory_arrays_rewriter.h"
 #include "theory/builtin/abstract_type.h"
+#include "theory/builtin/generic_op.h"
 #include "theory/datatypes/project_op.h"
 #include "theory/datatypes/sygus_datatype_utils.h"
 #include "theory/quantifiers/quantifiers_attributes.h"
@@ -213,6 +215,9 @@ void Smt2Printer::toStream(std::ostream& out,
       }
       break;
     }
+    case kind::APPLY_INDEXED_SYMBOLIC_OP:
+      out << smtKindString(n.getConst<GenericOp>().getKind());
+      break;
     case kind::BITVECTOR_TYPE:
       out << "(_ BitVec " << n.getConst<BitVectorSize>().d_size << ")";
       break;
@@ -360,6 +365,9 @@ void Smt2Printer::toStream(std::ostream& out,
       out << ss.str();
       break;
     }
+    case kind::DIVISIBLE_OP:
+      out << "(_ divisible " << n.getConst<Divisible>().k << ")";
+      break;
     case kind::SET_EMPTY:
       out << "(as set.empty ";
       toStreamType(out, n.getConst<EmptySet>().getType());
@@ -474,6 +482,33 @@ void Smt2Printer::toStream(std::ostream& out,
       out << "(_ re.loop " << n.getConst<RegExpLoop>().d_loopMinOcc << " "
           << n.getConst<RegExpLoop>().d_loopMaxOcc << ")";
       break;
+    case kind::TUPLE_PROJECT_OP:
+    case kind::TABLE_PROJECT_OP:
+    case kind::TABLE_AGGREGATE_OP:
+    case kind::TABLE_JOIN_OP:
+    case kind::TABLE_GROUP_OP:
+    case kind::RELATION_GROUP_OP:
+    case kind::RELATION_AGGREGATE_OP:
+    case kind::RELATION_PROJECT_OP:
+    {
+      ProjectOp op = n.getConst<ProjectOp>();
+      const std::vector<uint32_t>& indices = op.getIndices();
+      Kind k = NodeManager::operatorToKind(n);
+      if (indices.empty())
+      {
+        out << smtKindString(k);
+      }
+      else
+      {
+        out << "(_ " << smtKindString(k);
+        for (uint32_t i : indices)
+        {
+          out << " " << i;
+        }
+        out << ")";
+      }
+    }
+      break;
     default:
       // fall back on whatever operator<< does on underlying type; we
       // might luck out and be SMT-LIB v2 compliant
@@ -546,7 +581,7 @@ void Smt2Printer::toStream(std::ostream& out,
   {
     // abstract value
     std::string s = n.getName();
-    out << "(as @" << cvc5::internal::quoteSymbol(s) << " " << n.getType() << ")";
+    out << "(as " << cvc5::internal::quoteSymbol(s) << " " << n.getType() << ")";
     return;
   }
   else if (n.isVar())
@@ -589,7 +624,6 @@ void Smt2Printer::toStream(std::ostream& out,
   }
 
   bool stillNeedToPrintParams = true;
-  bool forceBinary = false; // force N-ary to binary when outputing children
   // operator
   if (n.getNumChildren() != 0 && k != kind::CONSTRUCTOR_TYPE)
   {
@@ -639,6 +673,9 @@ void Smt2Printer::toStream(std::ostream& out,
         out << ")";
       }
       return;
+    case kind::APPLY_INDEXED_SYMBOLIC:
+      // operator is printed as kind
+      break;
 
     case kind::MATCH:
       out << smtKindString(k) << " ";
@@ -701,7 +738,8 @@ void Smt2Printer::toStream(std::ostream& out,
       break;
 
     case kind::DIVISIBLE:
-      out << "(_ divisible " << n.getOperator().getConst<Divisible>().k << ")";
+      toStream(out, n.getOperator(), toDepth, nullptr);
+      out << ' ';
       stillNeedToPrintParams = false;
       break;
     case kind::REAL_ALGEBRAIC_NUMBER:
@@ -736,7 +774,6 @@ void Smt2Printer::toStream(std::ostream& out,
   case kind::BITVECTOR_ADD:
   {
     out << smtKindString(k) << " ";
-    forceBinary = true;
   }
   break;
 
@@ -811,126 +848,18 @@ void Smt2Printer::toStream(std::ostream& out,
     break;
   }
   case kind::TUPLE_PROJECT:
-  {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (tuple.project tuple)
-      out << "tuple.project " << n[0] << ")";
-    }
-    else
-    {
-      // e.g. ((_ tuple.project 2 4 4) tuple)
-      out << "(_ tuple.project" << op << ") " << n[0] << ")";
-    }
-    return;
-  }
   case kind::TABLE_PROJECT:
-  {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (table.project A)
-      out << "table.project " << n[0] << ")";
-    }
-    else
-    {
-      // e.g. ((_ table.project 2 4 4) A)
-      out << "(_ table.project" << op << ") " << n[0] << ")";
-    }
-    return;
-  }
   case kind::TABLE_AGGREGATE:
-  {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (table.aggr function initial_value bag)
-      out << "table.aggr " << n[0] << " " << n[1] << " " << n[2] << ")";
-    }
-    else
-    {
-      // e.g.  ((_ table.aggr 0) function initial_value bag)
-      out << "(_ table.aggr" << op << ") " << n[0] << " " << n[1] << " " << n[2]
-          << ")";
-    }
-    return;
-  }
   case kind::TABLE_JOIN:
-  {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (table.join A B)
-      out << "table.join " << n[0] << " " << n[1] << ")";
-    }
-    else
-    {
-      // e.g. ((_ table.project 0 1 2 3) A B)
-      out << "(_ table.join" << op << ") " << n[0] << " " << n[1] << ")";
-    }
-    return;
-  }
   case kind::TABLE_GROUP:
-  {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (table.group A)
-      out << "table.group " << n[0] << ")";
-    }
-    else
-    {
-      // e.g. ((_ table.group 0 1 2 3) A)
-      out << "(_ table.group" << op << ") " << n[0] << ")";
-    }
-    return;
-  }
   case kind::RELATION_GROUP:
-  {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (rel.group A)
-      out << "rel.group " << n[0] << ")";
-    }
-    else
-    {
-      // e.g. ((_ rel.group 0 1 2 3) A)
-      out << "(_ rel.group" << op << ") " << n[0] << ")";
-    }
-    return;
-  }
   case kind::RELATION_AGGREGATE:
-  {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (rel.aggr function initial_value bag)
-      out << "rel.aggr " << n[0] << " " << n[1] << " " << n[2] << ")";
-    }
-    else
-    {
-      // e.g.  ((_ rel.aggr 0) function initial_value bag)
-      out << "(_ rel.aggr" << op << ") " << n[0] << " " << n[1] << " " << n[2]
-          << ")";
-    }
-    return;
-  }
   case kind::RELATION_PROJECT:
   {
-    ProjectOp op = n.getOperator().getConst<ProjectOp>();
-    if (op.getIndices().empty())
-    {
-      // e.g. (rel.project A)
-      out << "rel.project " << n[0] << ")";
-    }
-    else
-    {
-      // e.g. ((_ rel.project 2 4 4) A)
-      out << "(_ rel.project" << op << ") " << n[0] << ")";
-    }
-    return;
+    toStream(out, n.getOperator(), toDepth, nullptr);
+    out << ' ';
+    stillNeedToPrintParams = false;
+    break;
   }
   case kind::CONSTRUCTOR_TYPE:
   {
@@ -1114,15 +1043,7 @@ void Smt2Printer::toStream(std::ostream& out,
       out << "(...)";
     }
     if(++i < n.getNumChildren()) {
-      if(forceBinary && i < n.getNumChildren() - 1) {
-        // not going to work properly for parameterized kinds!
-        Assert(n.getMetaKind() != kind::metakind::PARAMETERIZED);
-        out << " (" << smtKindStringOf(n) << ' ';
-        parens << ')';
-        ++c;
-      } else {
-        out << ' ';
-      }
+      out << ' ';
     }
   }
   if (n.getNumChildren() != 0)
@@ -1258,6 +1179,7 @@ std::string Smt2Printer::smtKindString(Kind k)
     case kind::APPLY_TESTER: return "is";
     case kind::APPLY_UPDATER: return "update";
     case kind::TUPLE_TYPE: return "Tuple";
+    case kind::TUPLE_PROJECT: return "tuple.project";
 
     // set theory
     case kind::SET_UNION: return "set.union";
@@ -1463,20 +1385,17 @@ std::string Smt2Printer::smtKindStringOf(const Node& n)
   return smtKindString(k);
 }
 
-void Smt2Printer::toStreamDeclareType(std::ostream& out, TypeNode tn) const
+void Smt2Printer::toStreamDeclareType(std::ostream& out,
+                                      const std::vector<TypeNode>& argTypes,
+                                      TypeNode tn) const
 {
   out << "(";
-  if (tn.isFunction())
+  if (!argTypes.empty())
   {
-    const vector<TypeNode> argTypes = tn.getArgTypes();
-    if (argTypes.size() > 0)
-    {
-      copy(argTypes.begin(),
-           argTypes.end() - 1,
-           ostream_iterator<TypeNode>(out, " "));
-      out << argTypes.back();
-    }
-    tn = tn.getRangeType();
+    copy(argTypes.begin(),
+         argTypes.end() - 1,
+         ostream_iterator<TypeNode>(out, " "));
+    out << argTypes.back();
   }
   out << ") " << tn;
 }
@@ -1545,7 +1464,7 @@ void Smt2Printer::toStreamModelSort(std::ostream& out,
   out << "; cardinality of " << tn << " is " << elements.size() << endl;
   if (modelUninterpPrint == options::ModelUninterpPrintMode::DeclSortAndFun)
   {
-    toStreamCmdDeclareType(out, tn);
+    Printer::toStreamCmdDeclareType(out, tn);
   }
   // print the representatives
   for (const Node& trn : elements)
@@ -1692,22 +1611,26 @@ void Smt2Printer::toStreamCmdQuit(std::ostream& out) const
   out << "(exit)" << std::endl;
 }
 
-void Smt2Printer::toStreamCmdDeclareFunction(std::ostream& out,
-                                             const std::string& id,
-                                             TypeNode type) const
+void Smt2Printer::toStreamCmdDeclareFunction(
+    std::ostream& out,
+    const std::string& id,
+    const std::vector<TypeNode>& argTypes,
+    TypeNode type) const
 {
   out << "(declare-fun " << cvc5::internal::quoteSymbol(id) << " ";
-  toStreamDeclareType(out, type);
+  toStreamDeclareType(out, argTypes, type);
   out << ')' << std::endl;
 }
 
-void Smt2Printer::toStreamCmdDeclareOracleFun(std::ostream& out,
-                                              const std::string& id,
-                                              TypeNode type,
-                                              const std::string& binName) const
+void Smt2Printer::toStreamCmdDeclareOracleFun(
+    std::ostream& out,
+    const std::string& id,
+    const std::vector<TypeNode>& argTypes,
+    TypeNode type,
+    const std::string& binName) const
 {
   out << "(declare-oracle-fun " << cvc5::internal::quoteSymbol(id) << " ";
-  toStreamDeclareType(out, type);
+  toStreamDeclareType(out, argTypes, type);
   out << " " << binName << ")" << std::endl;
 }
 
@@ -1818,13 +1741,11 @@ void Smt2Printer::toStreamSortedVarList(std::ostream& out,
 }
 
 void Smt2Printer::toStreamCmdDeclareType(std::ostream& out,
-                                         TypeNode type) const
+                                         const std::string& id,
+                                         size_t arity) const
 {
-  Assert(type.isUninterpretedSort() || type.isUninterpretedSortConstructor());
-  size_t arity = type.isUninterpretedSortConstructor()
-                     ? type.getUninterpretedSortConstructorArity()
-                     : 0;
-  out << "(declare-sort " << type << " " << arity << ")" << std::endl;
+  out << "(declare-sort " << cvc5::internal::quoteSymbol(id) << " " << arity
+      << ")" << std::endl;
 }
 
 void Smt2Printer::toStreamCmdDefineType(std::ostream& out,
@@ -1902,7 +1823,7 @@ void Smt2Printer::toStreamCmdGetProof(std::ostream& out,
                                       modes::ProofComponent c) const
 {
   out << "(get-proof";
-  if (c != modes::PROOF_COMPONENT_FULL)
+  if (c != modes::ProofComponent::FULL)
   {
     out << " :" << c;
   }
@@ -1924,11 +1845,16 @@ void Smt2Printer::toStreamCmdGetDifficulty(std::ostream& out) const
   out << "(get-difficulty)" << std::endl;
 }
 
+void Smt2Printer::toStreamCmdGetTimeoutCore(std::ostream& out) const
+{
+  out << "(get-timeout-core)" << std::endl;
+}
+
 void Smt2Printer::toStreamCmdGetLearnedLiterals(std::ostream& out,
                                                 modes::LearnedLitType t) const
 {
   out << "(get-learned-literals";
-  if (t != modes::LEARNED_LIT_INPUT)
+  if (t != modes::LearnedLitType::INPUT)
   {
     out << " :" << t;
   }
@@ -2093,38 +2019,45 @@ std::string Smt2Printer::sygusGrammarString(const TypeNode& t)
     {
       TypeNode curr = typesToPrint.front();
       typesToPrint.pop_front();
-      Assert(curr.isDatatype() && curr.getDType().isSygus());
+      // skip builtin fields, which can originate from any-constant constructors
+      if (!curr.isDatatype() || !curr.getDType().isSygus())
+      {
+        continue;
+      }
       const DType& dt = curr.getDType();
       types_list << '(' << dt.getName() << ' ' << dt.getSygusType() << " (";
       types_predecl << '(' << dt.getName() << ' ' << dt.getSygusType() << ") ";
-      if (dt.getSygusAllowConst())
-      {
-        types_list << "(Constant " << dt.getSygusType() << ") ";
-      }
       for (size_t i = 0, ncons = dt.getNumConstructors(); i < ncons; i++)
       {
         const DTypeConstructor& cons = dt[i];
-        // make a sygus term
-        std::vector<Node> cchildren;
-        cchildren.push_back(cons.getConstructor());
-        for (size_t j = 0, nargs = cons.getNumArgs(); j < nargs; j++)
+        if (cons.isSygusAnyConstant())
         {
-          TypeNode argType = cons[j].getRangeType();
-          std::stringstream ss;
-          ss << argType;
-          Node bv = nm->mkBoundVar(ss.str(), argType);
-          cchildren.push_back(bv);
-          // if fresh type, store it for later processing
-          if (grammarTypes.insert(argType).second)
+          types_list << "(Constant " << cons[0].getRangeType() << ") ";
+        }
+        else
+        {
+          // make a sygus term
+          std::vector<Node> cchildren;
+          cchildren.push_back(cons.getConstructor());
+          for (size_t j = 0, nargs = cons.getNumArgs(); j < nargs; j++)
           {
-            typesToPrint.push_back(argType);
+            TypeNode argType = cons[j].getRangeType();
+            std::stringstream ss;
+            ss << argType;
+            Node bv = nm->mkBoundVar(ss.str(), argType);
+            cchildren.push_back(bv);
+            // if fresh type, store it for later processing
+            if (grammarTypes.insert(argType).second)
+            {
+              typesToPrint.push_back(argType);
+            }
           }
+          Node consToPrint = nm->mkNode(kind::APPLY_CONSTRUCTOR, cchildren);
+          // now, print it using the conversion to builtin with external
+          types_list << theory::datatypes::utils::sygusToBuiltin(consToPrint,
+                                                                true);
+          types_list << ' ';
         }
-        Node consToPrint = nm->mkNode(kind::APPLY_CONSTRUCTOR, cchildren);
-        // now, print it using the conversion to builtin with external
-        types_list << theory::datatypes::utils::sygusToBuiltin(consToPrint,
-                                                               true);
-        types_list << ' ';
       }
       types_list << "))\n";
     } while (!typesToPrint.empty());
@@ -2135,21 +2068,16 @@ std::string Smt2Printer::sygusGrammarString(const TypeNode& t)
 }
 
 void Smt2Printer::toStreamCmdSynthFun(std::ostream& out,
-                                      Node f,
+                                      const std::string& id,
                                       const std::vector<Node>& vars,
-                                      bool isInv,
+                                      TypeNode rangeType,
                                       TypeNode sygusType) const
 {
-  out << '(' << (isInv ? "synth-inv " : "synth-fun ") << f << ' ';
+  out << "(synth-fun " << cvc5::internal::quoteSymbol(id) << ' ';
   // print variable list
   toStreamSortedVarList(out, vars);
-  // if not invariant-to-synthesize, print return type
-  if (!isInv)
-  {
-    TypeNode ftn = f.getType();
-    TypeNode range = ftn.isFunction() ? ftn.getRangeType() : ftn;
-    out << ' ' << range;
-  }
+  // print return type
+  out << ' ' << rangeType;
   out << '\n';
   // print grammar, if any
   if (!sygusType.isNull())
@@ -2160,10 +2088,11 @@ void Smt2Printer::toStreamCmdSynthFun(std::ostream& out,
 }
 
 void Smt2Printer::toStreamCmdDeclareVar(std::ostream& out,
-                                        Node var,
+                                        const std::string& id,
                                         TypeNode type) const
 {
-  out << "(declare-var " << var << ' ' << type << ')' << std::endl;
+  out << "(declare-var " << cvc5::internal::quoteSymbol(id) << ' ' << type
+      << ')' << std::endl;
 }
 
 void Smt2Printer::toStreamCmdConstraint(std::ostream& out, Node n) const
@@ -2193,6 +2122,24 @@ void Smt2Printer::toStreamCmdCheckSynthNext(std::ostream& out) const
   out << "(check-synth-next)" << std::endl;
 }
 
+void Smt2Printer::toStreamCmdFindSynth(std::ostream& out,
+                                       modes::FindSynthTarget fst,
+                                       TypeNode sygusType) const
+{
+  out << "(find-synth :" << fst;
+  // print grammar, if any
+  if (!sygusType.isNull())
+  {
+    out << " " << sygusGrammarString(sygusType);
+  }
+  out << ")" << std::endl;
+}
+
+void Smt2Printer::toStreamCmdFindSynthNext(std::ostream& out) const
+{
+  out << "(find-synth-next)" << std::endl;
+}
+
 void Smt2Printer::toStreamCmdGetInterpol(std::ostream& out,
                                          const std::string& name,
                                          Node conj,
