@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Abdalrhman Mohamed, Andrew Reynolds, Mathias Preiner
+ *   Abdalrhman Mohamed, Andrew Reynolds, Andres Noetzli
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -23,6 +23,7 @@
 namespace cvc5::internal {
 
 class LetBinding;
+class DType;
 
 namespace printer {
 namespace smt2 {
@@ -78,11 +79,13 @@ class Smt2Printer : public cvc5::internal::Printer
   /** Print declare-fun command */
   void toStreamCmdDeclareFunction(std::ostream& out,
                                   const std::string& id,
+                                  const std::vector<TypeNode>& argTypes,
                                   TypeNode type) const override;
 
   /** Print declare-oracle-fun command */
   void toStreamCmdDeclareOracleFun(std::ostream& out,
                                    const std::string& id,
+                                   const std::vector<TypeNode>& argTypes,
                                    TypeNode type,
                                    const std::string& binName) const override;
 
@@ -93,7 +96,9 @@ class Smt2Printer : public cvc5::internal::Printer
                                       const std::vector<Node>& initValue) const override;
 
   /** Print declare-sort command */
-  void toStreamCmdDeclareType(std::ostream& out, TypeNode type) const override;
+  void toStreamCmdDeclareType(std::ostream& out,
+                              const std::string& id,
+                              size_t arity) const override;
 
   /** Print define-sort command */
   void toStreamCmdDefineType(std::ostream& out,
@@ -127,16 +132,16 @@ class Smt2Printer : public cvc5::internal::Printer
 
   /** Print declare-var command */
   void toStreamCmdDeclareVar(std::ostream& out,
-                             Node var,
+                             const std::string& id,
                              TypeNode type) const override;
 
   /** Print synth-fun command */
   void toStreamCmdSynthFun(
       std::ostream& out,
-      Node f,
+      const std::string& id,
       const std::vector<Node>& vars,
-      bool isInv,
-      TypeNode sygusType = TypeNode::null()) const override;
+      TypeNode rangeType,
+      TypeNode sygusType) const override;
 
   /** Print constraint command */
   void toStreamCmdConstraint(std::ostream& out, Node n) const override;
@@ -157,6 +162,14 @@ class Smt2Printer : public cvc5::internal::Printer
   /** Print check-synth-next command */
   void toStreamCmdCheckSynthNext(std::ostream& out) const override;
 
+  /** Print find-synth command */
+  void toStreamCmdFindSynth(std::ostream& out,
+                            modes::FindSynthTarget fst,
+                            TypeNode sygusType) const override;
+
+  /** Print find-synth-next command */
+  void toStreamCmdFindSynthNext(std::ostream& out) const override;
+
   /** Print simplify command */
   void toStreamCmdSimplify(std::ostream& out, Node nodes) const override;
 
@@ -214,6 +227,9 @@ class Smt2Printer : public cvc5::internal::Printer
   /** Print get-difficulty command */
   void toStreamCmdGetDifficulty(std::ostream& out) const override;
 
+  /** Print get-timeout-core command */
+  void toStreamCmdGetTimeoutCore(std::ostream& out) const override;
+
   /** Print get-learned-literals command */
   void toStreamCmdGetLearnedLiterals(std::ostream& out,
                                      modes::LearnedLitType t) const override;
@@ -294,7 +310,9 @@ class Smt2Printer : public cvc5::internal::Printer
    * `() T` if the type T is not a function, or `(T1 ... Tn) Tr` if T is
    * a function type with argument types T1 ... Tn and return Tr.
    */
-  void toStreamDeclareType(std::ostream& out, TypeNode tn) const;
+  void toStreamDeclareType(std::ostream& out,
+                           const std::vector<TypeNode>& argTypes,
+                           TypeNode tn) const;
   /** To stream type node, which ensures tn is printed in smt2 format */
   void toStreamType(std::ostream& out, TypeNode tn) const;
   /** To stream datatype */
