@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -20,6 +20,7 @@
 #include "base/check.h"
 #include "base/output.h"
 #include "expr/node.h"
+#include "expr/skolem_manager.h"
 #include "options/bv_options.h"
 #include "printer/printer.h"
 #include "proof/clause_id.h"
@@ -185,6 +186,7 @@ SatLiteral CnfStream::newLiteral(TNode node,
     {
       Trace("cnf") << d_name << "::newLiteral: new var\n";
       lit = SatLiteral(d_satSolver->newVar(isTheoryAtom, canEliminate));
+      d_stats.d_numAtoms++;
     }
     d_nodeToLiteralMap.insert(node, lit);
     d_nodeToLiteralMap.insert(node.notNode(), ~lit);
@@ -256,8 +258,18 @@ SatLiteral CnfStream::convertAtom(TNode node)
   bool canEliminate = true;
   bool preRegister = false;
 
-  // Is this a variable add it to the list
-  if (node.isVar() && node.getKind() != kind::BOOLEAN_TERM_VARIABLE)
+  // Is this a variable add it to the list. We distinguish whether a Boolean
+  // variable has been marked as a "purification skolem". This is done
+  // by the term formula removal pass (term_formula_removal.h/cpp). We treat
+  // such variables as theory atoms since they may occur in term positions and
+  // thus need to be considered e.g. for theory combination.
+  bool isInternalBoolVar = false;
+  if (node.isVar())
+  {
+    SkolemManager* sm = NodeManager::currentNM()->getSkolemManager();
+    isInternalBoolVar = (sm->getId(node) != SkolemFunId::PURIFY);
+  }
+  if (isInternalBoolVar)
   {
     d_booleanVariables.push_back(node);
     // if TRACK_AND_NOTIFY_VAR, we are notified when Boolean variables are
@@ -761,7 +773,8 @@ void CnfStream::convertAndAssert(TNode node, bool negated)
 CnfStream::Statistics::Statistics(StatisticsRegistry& sr,
                                   const std::string& name)
     : d_cnfConversionTime(
-        sr.registerTimer(name + "::CnfStream::cnfConversionTime"))
+        sr.registerTimer(name + "::CnfStream::cnfConversionTime")),
+      d_numAtoms(sr.registerInt(name + "::CnfStream::numAtoms"))
 {
 }
 
