@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -86,7 +86,7 @@ class MinisatSatSolver : public CDCLTSatSolver, protected EnvObj
 
   void resetTrail() override;
 
-  void requirePhase(SatLiteral lit) override;
+  void preferPhase(SatLiteral lit) override;
 
   bool isDecision(SatVariable decn) const override;
 
