@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -45,32 +45,28 @@ void PrintBenchmark::printAssertions(std::ostream& out,
   {
     // note that we must get all "component types" of a type, so that
     // e.g. U is printed as a sort declaration when we have type (Array U Int).
-    std::unordered_set<TypeNode> ctypes;
-    expr::getComponentTypes(st, ctypes);
-    for (const TypeNode& stc : ctypes)
+    // get all connected datatypes to this one
+    std::vector<TypeNode> connectedTypes;
+    getConnectedSubfieldTypes(st, connectedTypes, alreadyPrintedDeclSorts);
+    // now, separate into sorts and datatypes
+    std::vector<TypeNode> datatypeBlock;
+    for (const TypeNode& ctn : connectedTypes)
     {
-      // get all connected datatypes to this one
-      std::vector<TypeNode> connectedTypes;
-      getConnectedSubfieldTypes(stc, connectedTypes, alreadyPrintedDeclSorts);
-      // now, separate into sorts and datatypes
-      std::vector<TypeNode> datatypeBlock;
-      for (const TypeNode& ctn : connectedTypes)
+      if ((ctn.isUninterpretedSort() && ctn.getNumChildren() == 0)
+          || ctn.isUninterpretedSortConstructor())
       {
-        if (ctn.isUninterpretedSort())
-        {
-          d_printer->toStreamCmdDeclareType(out, ctn);
-        }
-        else if (ctn.isDatatype())
-        {
-          datatypeBlock.push_back(ctn);
-        }
+        d_printer->toStreamCmdDeclareType(out, ctn);
       }
-      // print the mutually recursive datatype block if necessary
-      if (!datatypeBlock.empty())
+      else if (ctn.isDatatype())
       {
-        d_printer->toStreamCmdDatatypeDeclaration(out, datatypeBlock);
+        datatypeBlock.push_back(ctn);
       }
     }
+    // print the mutually recursive datatype block if necessary
+    if (!datatypeBlock.empty())
+    {
+      d_printer->toStreamCmdDatatypeDeclaration(out, datatypeBlock);
+    }
   }
 
   // global visited cache for expr::getSymbols calls
@@ -183,19 +179,34 @@ void PrintBenchmark::getConnectedSubfieldTypes(
     return;
   }
   processed.insert(tn);
-  if (tn.isUninterpretedSort())
+  if (tn.isParametricDatatype())
   {
-    connectedTypes.push_back(tn);
-  }
-  else if (tn.isDatatype())
-  {
-    connectedTypes.push_back(tn);
-    std::unordered_set<TypeNode> subfieldTypes =
-        tn.getDType().getSubfieldTypes();
-    for (const TypeNode& ctn : subfieldTypes)
+    const DType& dt = tn.getDType();
+    // ignore its parameters
+    for (size_t i = 0, nparams = dt.getNumParameters(); i < nparams; i++)
     {
-      getConnectedSubfieldTypes(ctn, connectedTypes, processed);
+      processed.insert(dt.getParameter(i));
     }
+    // we do not process the datatype here, instead we will traverse to the
+    // head of the parameteric datatype (tn[0]), which will subsequently
+    // process its subfield types.
+  }
+  else
+  {
+    connectedTypes.push_back(tn);
+    if (tn.isDatatype())
+    {
+      std::unordered_set<TypeNode> subfieldTypes =
+          tn.getDType().getSubfieldTypes();
+      for (const TypeNode& ctn : subfieldTypes)
+      {
+        getConnectedSubfieldTypes(ctn, connectedTypes, processed);
+      }
+    }
+  }
+  for (unsigned i = 0, nchild = tn.getNumChildren(); i < nchild; i++)
+  {
+    getConnectedSubfieldTypes(tn[i], connectedTypes, processed);
   }
 }
 
