@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -46,11 +46,10 @@ SineSolver::SineSolver(Env& env, TranscendentalState* tstate)
   Node one = nm->mkConstReal(Rational(1));
   Node negOne = nm->mkConstReal(Rational(-1));
   d_pi = nm->mkNullaryOperator(nm->realType(), Kind::PI);
-  Node pi_2 = rewrite(
-      nm->mkNode(Kind::MULT, d_pi, nm->mkConstReal(Rational(1) / Rational(2))));
-  Node pi_neg_2 = rewrite(nm->mkNode(
-      Kind::MULT, d_pi, nm->mkConstReal(Rational(-1) / Rational(2))));
-  d_neg_pi = rewrite(nm->mkNode(Kind::MULT, d_pi, negOne));
+  Node pi_2 = nm->mkNode(Kind::MULT, nm->mkConstReal(Rational(1, 2)), d_pi);
+  Node pi_neg_2 =
+      nm->mkNode(Kind::MULT, nm->mkConstReal(Rational(-1, 2)), d_pi);
+  d_neg_pi = nm->mkNode(Kind::MULT, nm->mkConstInt(Rational(-1)), d_pi);
   d_mpoints.push_back(d_pi);
   d_mpointsSine[d_pi] = zero;
   d_mpoints.push_back(pi_2);
@@ -225,6 +224,7 @@ void SineSolver::checkInitialRefine()
       // initial refinements
       if (d_tf_initial_refine.find(t) == d_tf_initial_refine.end())
       {
+        Trace("nl-ext-debug") << "Process initial refine " << t << std::endl;
         d_tf_initial_refine[t] = true;
         Assert(d_data->isPurified(t));
         {
