@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Alex Ozdemir, Aina Niemetz
+ *   Alex Ozdemir, Andrew Reynolds, Aina Niemetz
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -20,6 +20,9 @@
 
 #include "expr/node.h"
 #include "proof/proof_checker.h"
+#include "theory/arith/nl/coverings/proof_checker.h"
+#include "theory/arith/nl/ext/proof_checker.h"
+#include "theory/arith/nl/transcendental/proof_checker.h"
 
 namespace cvc5::internal {
 namespace theory {
@@ -29,7 +32,7 @@ namespace arith {
 class ArithProofRuleChecker : public ProofRuleChecker
 {
  public:
-  ArithProofRuleChecker() {}
+  ArithProofRuleChecker();
   ~ArithProofRuleChecker() {}
 
   /** Register all rules owned by this rule checker into pc. */
@@ -40,6 +43,14 @@ class ArithProofRuleChecker : public ProofRuleChecker
   Node checkInternal(PfRule id,
                      const std::vector<Node>& children,
                      const std::vector<Node>& args) override;
+  /** The proof checker for proofs of the nlext. */
+  nl::ExtProofRuleChecker d_extChecker;
+  /** The proof checker for transcendental proofs */
+  nl::transcendental::TranscendentalProofRuleChecker d_trChecker;
+#ifdef CVC5_POLY_IMP
+  /** The proof checker for coverings proofs */
+  nl::coverings::CoveringsProofRuleChecker d_covChecker;
+#endif
 };
 
 }  // namespace arith
