@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Aina Niemetz, Mathias Preiner, Clark Barrett
+ *   Andrew Reynolds, Aina Niemetz, Clark Barrett
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -26,30 +26,51 @@ namespace cvc5::internal {
 namespace theory {
 namespace arrays {
 
+TypeNode ArraySelectTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
+
 TypeNode ArraySelectTypeRule::computeType(NodeManager* nodeManager,
                                           TNode n,
                                           bool check,
                                           std::ostream* errOut)
 {
   Assert(n.getKind() == kind::SELECT);
-  TypeNode arrayType = n[0].getType(check);
+  TypeNode arrayType = n[0].getTypeOrNull();
   if (check)
   {
-    if (!arrayType.isArray())
+    if (!arrayType.isMaybeKind(kind::ARRAY_TYPE))
     {
-      throw TypeCheckingExceptionPrivate(n,
-                                         "array select operating on non-array");
+      if (errOut)
+      {
+        (*errOut) << "array select operating on non-array";
+      }
+      return TypeNode::null();
     }
-    TypeNode indexType = n[1].getType(check);
-    if (indexType != arrayType.getArrayIndexType())
+    TypeNode indexType = n[1].getTypeOrNull();
+    if (!indexType.isComparableTo(arrayType.getArrayIndexType()))
     {
-      throw TypeCheckingExceptionPrivate(
-          n, "array select not indexed with correct type for array");
+      if (errOut)
+      {
+        (*errOut) << "array select not indexed with correct type for array";
+      }
+      return TypeNode::null();
     }
   }
+  if (arrayType.isAbstract())
+  {
+    // if selecting from a (fully) abstract array, the return is unknown.
+    return nodeManager->mkAbstractType(kind::ABSTRACT_TYPE);
+  }
+  // otherwise
   return arrayType.getArrayConstituentType();
 }
 
+TypeNode ArrayStoreTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ArrayStoreTypeRule::computeType(NodeManager* nodeManager,
                                          TNode n,
                                          bool check,
@@ -57,32 +78,41 @@ TypeNode ArrayStoreTypeRule::computeType(NodeManager* nodeManager,
 {
   if (n.getKind() == kind::STORE)
   {
-    TypeNode arrayType = n[0].getType(check);
+    TypeNode arrayType = n[0].getTypeOrNull();
     if (check)
     {
-      if (!arrayType.isArray())
+      if (!arrayType.isMaybeKind(kind::ARRAY_TYPE))
       {
-        throw TypeCheckingExceptionPrivate(
-            n, "array store operating on non-array");
-      }
-      TypeNode indexType = n[1].getType(check);
-      TypeNode valueType = n[2].getType(check);
-      if (indexType != arrayType.getArrayIndexType())
-      {
-        throw TypeCheckingExceptionPrivate(
-            n, "array store not indexed with correct type for array");
-      }
-      if (valueType != arrayType.getArrayConstituentType())
-      {
-        Trace("array-types")
-            << "array type: " << arrayType.getArrayConstituentType()
-            << std::endl;
-        Trace("array-types") << "value types: " << valueType << std::endl;
-        throw TypeCheckingExceptionPrivate(
-            n, "array store not assigned with correct type for array");
+        if (errOut)
+        {
+          (*errOut) << "array store operating on non-array";
+        }
+        return TypeNode::null();
       }
     }
-    return arrayType;
+    TypeNode indexType = n[1].getTypeOrNull();
+    TypeNode aindexType = arrayType.getArrayIndexType();
+    TypeNode indexjoin = indexType.leastUpperBound(aindexType);
+    if (indexjoin.isNull())
+    {
+      if (errOut)
+      {
+        (*errOut) << "array store not indexed with correct type for array";
+      }
+      return TypeNode::null();
+    }
+    TypeNode valueType = n[2].getTypeOrNull();
+    TypeNode avalueType = arrayType.getArrayConstituentType();
+    TypeNode valuejoin = valueType.leastUpperBound(avalueType);
+    if (valuejoin.isNull())
+    {
+      if (errOut)
+      {
+        (*errOut) << "array store not assigned with correct type for array";
+      }
+      return TypeNode::null();
+    }
+    return NodeManager::currentNM()->mkArrayType(indexjoin, valuejoin);
   }
   else
   {
@@ -133,8 +163,16 @@ bool ArrayStoreTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
     return false;
   }
 
+  TypeNode itype = index.getType();
+  CardinalityClass itcc = itype.getCardinalityClass();
+
+  if (itcc != CardinalityClass::FINITE && itcc != CardinalityClass::ONE)
+  {
+    return true;
+  }
+
   // Get the cardinality of the index type
-  Cardinality indexCard = index.getType().getCardinality();
+  Cardinality indexCard = itype.getCardinality();
 
   if (indexCard.isInfinite())
   {
@@ -179,24 +217,35 @@ bool ArrayStoreTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
   return true;
 }
 
+TypeNode ArrayLambdaTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ArrayLambdaTypeRule::computeType(NodeManager* nodeManager,
                                           TNode n,
                                           bool check,
                                           std::ostream* errOut)
 {
   Assert(n.getKind() == kind::ARRAY_LAMBDA);
-  TypeNode lamType = n[0].getType(check);
+  TypeNode lamType = n[0].getTypeOrNull();
   if (check)
   {
     if (n[0].getKind() != kind::LAMBDA)
     {
-      throw TypeCheckingExceptionPrivate(n, "array lambda arg is non-lambda");
+      if (errOut)
+      {
+        (*errOut) << "array lambda arg is non-lambda";
+      }
+      return TypeNode::null();
     }
   }
   if (lamType.getNumChildren() != 2)
   {
-    throw TypeCheckingExceptionPrivate(n,
-                                       "array lambda arg is not unary lambda");
+    if (errOut)
+    {
+      (*errOut) << "array lambda arg is not unary lambda";
+    }
+    return TypeNode::null();
   }
   return nodeManager->mkArrayType(lamType[0], lamType[1]);
 }
@@ -239,6 +288,11 @@ Node ArraysProperties::mkGroundTerm(TypeNode type)
   return builtin::SortProperties::mkGroundTerm(type);
 }
 
+TypeNode ArrayEqRangeTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
+
 TypeNode ArrayEqRangeTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
                                            bool check,
@@ -247,42 +301,61 @@ TypeNode ArrayEqRangeTypeRule::computeType(NodeManager* nodeManager,
   Assert(n.getKind() == kind::EQ_RANGE);
   if (check)
   {
-    TypeNode n0_type = n[0].getType(check);
-    TypeNode n1_type = n[1].getType(check);
-    if (!n0_type.isArray())
+    TypeNode n0_type = n[0].getTypeOrNull();
+    TypeNode n1_type = n[1].getTypeOrNull();
+    if (!n0_type.isMaybeKind(kind::ARRAY_TYPE))
     {
-      throw TypeCheckingExceptionPrivate(
-          n, "first operand of eqrange is not an array");
+      if (errOut)
+      {
+        (*errOut) << "first operand of eqrange is not an array";
+      }
+      return TypeNode::null();
     }
-    if (!n1_type.isArray())
+    if (!n1_type.isMaybeKind(kind::ARRAY_TYPE))
     {
-      throw TypeCheckingExceptionPrivate(
-          n, "second operand of eqrange is not an array");
+      if (errOut)
+      {
+        (*errOut) << "second operand of eqrange is not an array";
+      }
+      return TypeNode::null();
     }
-    if (n0_type != n1_type)
+    if (!n0_type.isComparableTo(n1_type))
     {
-      throw TypeCheckingExceptionPrivate(n, "array types do not match");
+      if (errOut)
+      {
+        (*errOut) << "array types do not match";
+      }
+      return TypeNode::null();
     }
     TypeNode indexType = n0_type.getArrayIndexType();
-    TypeNode indexRangeType1 = n[2].getType(check);
-    TypeNode indexRangeType2 = n[3].getType(check);
-    if (indexRangeType1 != indexType)
+    TypeNode indexRangeType1 = n[2].getTypeOrNull();
+    TypeNode indexRangeType2 = n[3].getTypeOrNull();
+    if (!indexRangeType1.isComparableTo(indexType))
     {
-      throw TypeCheckingExceptionPrivate(
-          n, "eqrange lower index type does not match array index type");
+      if (errOut)
+      {
+        (*errOut) << "eqrange lower index type does not match array index type";
+      }
+      return TypeNode::null();
     }
-    if (indexRangeType2 != indexType)
+    if (!indexRangeType2.isComparableTo(indexType))
     {
-      throw TypeCheckingExceptionPrivate(
-          n, "eqrange upper index type does not match array index type");
+      if (errOut)
+      {
+        (*errOut) << "eqrange upper index type does not match array index type";
+      }
+      return TypeNode::null();
     }
-    if (!indexType.isBitVector() && !indexType.isFloatingPoint()
+    if (!indexType.isMaybeKind(kind::BITVECTOR_TYPE)
+        && !indexType.isMaybeKind(kind::FLOATINGPOINT_TYPE)
         && !indexType.isRealOrInt())
     {
-      throw TypeCheckingExceptionPrivate(
-          n,
-          "eqrange only supports bit-vectors, floating-points, integers, and "
-          "reals as index type");
+      if (errOut)
+      {
+        (*errOut) << "eqrange only supports bit-vectors, floating-points, "
+                     "integers, and reals as index type";
+      }
+      return TypeNode::null();
     }
   }
   return nodeManager->booleanType();
