@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Mudathir Mohamed, Aina Niemetz, Andres Noetzli
+ *   Mudathir Mohamed, Andrew Reynolds, Aina Niemetz
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -33,6 +33,10 @@ namespace bags {
 
 using namespace datatypes;
 
+TypeNode BinaryOperatorTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode BinaryOperatorTypeRule::computeType(NodeManager* nodeManager,
                                              TNode n,
                                              bool check,
@@ -72,6 +76,10 @@ bool BinaryOperatorTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
   return BagsUtils::isConstant(n);
 }
 
+TypeNode SubBagTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode SubBagTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -95,6 +103,10 @@ TypeNode SubBagTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode CountTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->integerType();
+}
 TypeNode CountTypeRule::computeType(NodeManager* nodeManager,
                                     TNode n,
                                     bool check,
@@ -125,6 +137,10 @@ TypeNode CountTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->integerType();
 }
 
+TypeNode MemberTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode MemberTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -155,6 +171,10 @@ TypeNode MemberTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode DuplicateRemovalTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode DuplicateRemovalTypeRule::computeType(NodeManager* nodeManager,
                                                TNode n,
                                                bool check,
@@ -175,6 +195,10 @@ TypeNode DuplicateRemovalTypeRule::computeType(NodeManager* nodeManager,
   return bagType;
 }
 
+TypeNode BagMakeTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode BagMakeTypeRule::computeType(NodeManager* nm,
                                       TNode n,
                                       bool check,
@@ -212,6 +236,10 @@ bool BagMakeTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
          && n[1].getConst<Rational>().sgn() == 1;
 }
 
+TypeNode IsSingletonTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode IsSingletonTypeRule::computeType(NodeManager* nodeManager,
                                           TNode n,
                                           bool check,
@@ -230,6 +258,10 @@ TypeNode IsSingletonTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode EmptyBagTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode EmptyBagTypeRule::computeType(NodeManager* nodeManager,
                                        TNode n,
                                        bool check,
@@ -240,6 +272,10 @@ TypeNode EmptyBagTypeRule::computeType(NodeManager* nodeManager,
   return emptyBag.getType();
 }
 
+TypeNode CardTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->integerType();
+}
 TypeNode CardTypeRule::computeType(NodeManager* nodeManager,
                                    TNode n,
                                    bool check,
@@ -258,6 +294,10 @@ TypeNode CardTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->integerType();
 }
 
+TypeNode ChooseTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ChooseTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -276,6 +316,10 @@ TypeNode ChooseTypeRule::computeType(NodeManager* nodeManager,
   return bagType.getBagElementType();
 }
 
+TypeNode FromSetTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode FromSetTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
                                       bool check,
@@ -296,6 +340,10 @@ TypeNode FromSetTypeRule::computeType(NodeManager* nodeManager,
   return bagType;
 }
 
+TypeNode ToSetTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ToSetTypeRule::computeType(NodeManager* nodeManager,
                                     TNode n,
                                     bool check,
@@ -316,6 +364,10 @@ TypeNode ToSetTypeRule::computeType(NodeManager* nodeManager,
   return setType;
 }
 
+TypeNode BagMapTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode BagMapTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -359,6 +411,10 @@ TypeNode BagMapTypeRule::computeType(NodeManager* nodeManager,
   return retType;
 }
 
+TypeNode BagFilterTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode BagFilterTypeRule::computeType(NodeManager* nodeManager,
                                         TNode n,
                                         bool check,
@@ -402,6 +458,10 @@ TypeNode BagFilterTypeRule::computeType(NodeManager* nodeManager,
   return bagType;
 }
 
+TypeNode BagFoldTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode BagFoldTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
                                       bool check,
@@ -454,6 +514,10 @@ TypeNode BagFoldTypeRule::computeType(NodeManager* nodeManager,
   return retType;
 }
 
+TypeNode BagPartitionTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode BagPartitionTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
                                            bool check,
@@ -499,6 +563,10 @@ TypeNode BagPartitionTypeRule::computeType(NodeManager* nodeManager,
   return retType;
 }
 
+TypeNode TableProductTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode TableProductTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
                                            bool check,
@@ -537,6 +605,10 @@ TypeNode TableProductTypeRule::computeType(NodeManager* nodeManager,
   return retType;
 }
 
+TypeNode TableProjectTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode TableProjectTypeRule::computeType(NodeManager* nm,
                                            TNode n,
                                            bool check,
@@ -595,6 +667,10 @@ TypeNode TableProjectTypeRule::computeType(NodeManager* nm,
   return nm->mkBagType(retTupleType);
 }
 
+TypeNode TableAggregateTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode TableAggregateTypeRule::computeType(NodeManager* nm,
                                              TNode n,
                                              bool check,
@@ -662,6 +738,10 @@ TypeNode TableAggregateTypeRule::computeType(NodeManager* nm,
   return nm->mkBagType(functionType.getRangeType());
 }
 
+TypeNode TableJoinTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode TableJoinTypeRule::computeType(NodeManager* nm,
                                         TNode n,
                                         bool check,
@@ -732,6 +812,10 @@ TypeNode TableJoinTypeRule::computeType(NodeManager* nm,
   return nm->mkBagType(retTupleType);
 }
 
+TypeNode TableGroupTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode TableGroupTypeRule::computeType(NodeManager* nm,
                                          TNode n,
                                          bool check,
