@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Mathias Preiner, Andres Noetzli, Andrew Reynolds
+ *   Mathias Preiner, Andrew Reynolds, Andres Noetzli
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -134,6 +134,13 @@ BVSolverBitblast::BVSolverBitblast(Env& env,
   initSatSolver();
 }
 
+bool BVSolverBitblast::needsEqualityEngine(EeSetupInfo& esi)
+{
+  // we always need the equality engine if sharing is enabled for processing
+  // equality engine and shared terms
+  return logicInfo().isSharingEnabled() || options().bv.bvEqEngine;
+}
+
 void BVSolverBitblast::postCheck(Theory::Effort level)
 {
   if (level != Theory::Effort::EFFORT_FULL)
@@ -257,8 +264,10 @@ bool BVSolverBitblast::preNotifyFact(
   {
     d_bbFacts.push_back(fact);
   }
-
-  return false;  // Return false to enable equality engine reasoning in Theory.
+  
+  // Return false to enable equality engine reasoning in Theory, which is
+  // available if we are using the equality engine.
+  return !logicInfo().isSharingEnabled() && !options().bv.bvEqEngine;
 }
 
 TrustNode BVSolverBitblast::explain(TNode n)
@@ -311,7 +320,7 @@ bool BVSolverBitblast::collectModelValues(TheoryModel* m,
     {
       Assert(d_cnfStream->hasLiteral(var));
       prop::SatLiteral bit = d_cnfStream->getLiteral(var);
-      prop::SatValue value = d_satSolver->value(bit);
+      prop::SatValue value = d_satSolver->modelValue(bit);
       Assert(value != prop::SAT_VALUE_UNKNOWN);
       if (!m->assertEquality(
               var, nm->mkConst(value == prop::SAT_VALUE_TRUE), true))
@@ -328,7 +337,7 @@ void BVSolverBitblast::initSatSolver()
 {
   switch (options().bv.bvSatSolver)
   {
-    case options::SatSolverMode::CRYPTOMINISAT:
+    case options::BvSatSolverMode::CRYPTOMINISAT:
       d_satSolver.reset(prop::SatSolverFactory::createCryptoMinisat(
           statisticsRegistry(),
           d_env.getResourceManager(),
