@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -318,7 +318,7 @@ RewriteResponse TheoryBVRewriter::RewriteXor(TNode node, bool prerewrite)
   if (!prerewrite)
   {
     resultNode =
-        LinearRewriteStrategy<RewriteRule<XorOne>,
+        LinearRewriteStrategy<RewriteRule<XorOnes>,
                               RewriteRule<BitwiseSlicing>>::apply(resultNode);
 
     if (resultNode.getKind() != node.getKind())
