@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Andrew Reynolds, Aina Niemetz, Mudathir Mohamed
+ *   Andrew Reynolds, Tim King, Aina Niemetz
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -33,6 +33,7 @@ namespace datatypes {
  * for type substitution sigma, and returns T * sigma.
  */
 struct DatatypeConstructorTypeRule {
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -51,6 +52,7 @@ struct DatatypeConstructorTypeRule {
  * This rule is generalized for parametric datatypes.
  */
 struct DatatypeSelectorTypeRule {
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -63,6 +65,7 @@ struct DatatypeSelectorTypeRule {
  * This rule is generalized for parametric datatypes.
  */
 struct DatatypeTesterTypeRule {
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -76,6 +79,7 @@ struct DatatypeTesterTypeRule {
  */
 struct DatatypeUpdateTypeRule
 {
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -89,6 +93,7 @@ struct DatatypeUpdateTypeRule
  */
 struct DatatypeAscriptionTypeRule
 {
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -105,6 +110,7 @@ struct ConstructorProperties
  */
 class DtSizeTypeRule {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -117,6 +123,7 @@ class DtSizeTypeRule {
  */
 class DtBoundTypeRule {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -132,6 +139,7 @@ class DtBoundTypeRule {
 class DtSygusEvalTypeRule
 {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -158,6 +166,7 @@ class DtSygusEvalTypeRule
 class MatchTypeRule
 {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -171,6 +180,7 @@ class MatchTypeRule
 class MatchCaseTypeRule
 {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -185,6 +195,7 @@ class MatchCaseTypeRule
 class MatchBindCaseTypeRule
 {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
@@ -200,6 +211,7 @@ class MatchBindCaseTypeRule
 class TupleProjectTypeRule
 {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nm,
                               TNode n,
                               bool check,
@@ -213,6 +225,7 @@ class TupleProjectTypeRule
 class CodatatypeBoundVariableTypeRule
 {
  public:
+  static TypeNode preComputeType(NodeManager* nm, TNode n);
   static TypeNode computeType(NodeManager* nodeManager,
                               TNode n,
                               bool check,
