@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Martin Brain, Andrew Reynolds, Aina Niemetz
+ *   Martin Brain, Aina Niemetz, Andrew Reynolds
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -17,6 +17,7 @@
 
 #include <set>
 #include <stack>
+#include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
@@ -62,15 +63,15 @@ Node buildConjunct(const std::vector<TNode> &assumptions) {
 /** Constructs a new instance of TheoryFp w.r.t. the provided contexts. */
 TheoryFp::TheoryFp(Env& env, OutputChannel& out, Valuation valuation)
     : Theory(THEORY_FP, env, out, valuation),
-      d_registeredTerms(userContext()),
       d_wordBlaster(new FpWordBlaster(userContext())),
-      d_expansionRequested(false),
+      d_registeredTerms(userContext()),
       d_abstractionMap(userContext()),
       d_rewriter(userContext()),
       d_state(env, valuation),
       d_im(env, *this, d_state, "theory::fp::", true),
       d_notify(d_im),
       d_wbFactsCache(userContext()),
+      d_invalidateModelCache(context(), true),
       d_true(NodeManager::currentNM()->mkConst(true))
 {
   // indicate we are using the default theory state and inference manager
@@ -549,7 +550,7 @@ void TheoryFp::registerTerm(TNode node)
     // Purify (fp.to_real x)
     NodeManager* nm = NodeManager::currentNM();
     SkolemManager* sm = nm->getSkolemManager();
-    Node sk = sm->mkPurifySkolem(node, "to_real", "fp purify skolem");
+    Node sk = sm->mkPurifySkolem(node);
     handleLemma(node.eqNode(sk), InferenceId::FP_REGISTER_TERM);
     d_abstractionMap.insert(sk, node);
 
@@ -575,7 +576,7 @@ void TheoryFp::registerTerm(TNode node)
     // Purify ((_ to_fp eb sb) rm x)
     NodeManager* nm = NodeManager::currentNM();
     SkolemManager* sm = nm->getSkolemManager();
-    Node sk = sm->mkPurifySkolem(node, "to_real_fp", "fp purify skolem");
+    Node sk = sm->mkPurifySkolem(node);
     handleLemma(node.eqNode(sk), InferenceId::FP_REGISTER_TERM);
     d_abstractionMap.insert(sk, node);
 
@@ -671,6 +672,8 @@ bool TheoryFp::needsCheckLastEffort()
 
 void TheoryFp::postCheck(Effort level)
 {
+  d_invalidateModelCache = true;
+
   /* Resolve the abstractions for the conversion lemmas */
   if (level == EFFORT_LAST_CALL)
   {
@@ -744,6 +747,101 @@ void TheoryFp::notifySharedTerm(TNode n)
   }
 }
 
+Node TheoryFp::getCandidateModelValue(TNode node)
+{
+  if (d_invalidateModelCache.get())
+  {
+    d_modelCache.clear();
+  }
+  d_invalidateModelCache.set(false);
+
+  std::vector<TNode> visit;
+  std::unordered_map<TNode, bool> visited;
+
+  TNode cur;
+  visit.push_back(node);
+  do
+  {
+    cur = visit.back();
+    visit.pop_back();
+
+    auto it = d_modelCache.find(cur);
+    if (it != d_modelCache.end() && !it->second.isNull())
+    {
+      continue;
+    }
+
+    auto vit = visited.find(cur);
+    if (vit != visited.end() && vit->second)
+    {
+      continue;
+    }
+
+    if (cur.isConst())
+    {
+      d_modelCache[cur] = cur;
+      visited[cur] = true;
+      continue;
+    }
+
+    Node value;
+
+    Kind kind = cur.getKind();
+    if (kind == kind::FLOATINGPOINT_TO_FP_FROM_SBV
+        || kind == kind::FLOATINGPOINT_TO_FP_FROM_UBV
+        || kind == kind::FLOATINGPOINT_TO_FP_FROM_REAL
+        || kind == kind::FLOATINGPOINT_TO_FP_FROM_IEEE_BV
+        || Theory::isLeafOf(cur, theory::THEORY_FP))
+    {
+      if (cur.getType().isFloatingPoint() || cur.getType().isRoundingMode())
+      {
+        value = d_wordBlaster->getValue(d_valuation, cur);
+      }
+      else
+      {
+        value = d_valuation.getCandidateModelValue(cur);
+        if (value.isNull())
+        {
+          return value;
+        }
+      }
+      d_modelCache[cur] = value;
+      visited[cur] = true;
+      continue;
+    }
+
+    if (vit == visited.end())
+    {
+      visit.push_back(cur);
+      visited.emplace(cur, false);
+      visit.insert(visit.end(), cur.begin(), cur.end());
+    }
+    else if (!vit->second)
+    {
+      NodeBuilder nb(kind);
+      if (cur.getMetaKind() == kind::metakind::PARAMETERIZED)
+      {
+        nb << cur.getOperator();
+      }
+
+      std::unordered_map<Node, Node>::iterator iit;
+      for (const TNode& child : cur)
+      {
+        iit = d_modelCache.find(child);
+        Assert(iit != d_modelCache.end());
+        Assert(!iit->second.isNull());
+        nb << iit->second;
+      }
+      d_modelCache[cur] = rewrite(nb.constructNode());
+      vit->second = true;
+    }
+  } while (!visit.empty());
+
+  auto it = d_modelCache.find(node);
+  Assert(it != d_modelCache.end());
+  return it->second;
+}
+
 TrustNode TheoryFp::explain(TNode n)
 {
   Trace("fp") << "TheoryFp::explain(): explain " << n << std::endl;
@@ -764,9 +862,21 @@ TrustNode TheoryFp::explain(TNode n)
   return TrustNode::mkTrustPropExp(n, exp, nullptr);
 }
 
-Node TheoryFp::getCandidateModelValue(TNode var)
+EqualityStatus TheoryFp::getEqualityStatus(TNode a, TNode b)
 {
-  return d_wordBlaster->getValue(d_valuation, var);
+  Node value_a = getCandidateModelValue(a);
+  Node value_b = getCandidateModelValue(b);
+  if (value_a.isNull() || value_b.isNull())
+  {
+    return EqualityStatus::EQUALITY_UNKNOWN;
+  }
+  if (value_a == value_b)
+  {
+    Trace("theory-fp") << EqualityStatus::EQUALITY_TRUE_IN_MODEL << std::endl;
+    return EqualityStatus::EQUALITY_TRUE_IN_MODEL;
+  }
+  Trace("theory-fp") << EqualityStatus::EQUALITY_FALSE_IN_MODEL << std::endl;
+  return EqualityStatus::EQUALITY_FALSE_IN_MODEL;
 }
 
 bool TheoryFp::collectModelInfo(TheoryModel* m,
@@ -777,18 +887,22 @@ bool TheoryFp::collectModelInfo(TheoryModel* m,
 }
 
 bool TheoryFp::collectModelValues(TheoryModel* m,
-                                  const std::set<Node>& relevantTerms)
+                                  const std::set<Node>& termSet)
 {
-  Trace("fp-collectModelInfo")
-      << "TheoryFp::collectModelInfo(): begin" << std::endl;
-  if (TraceIsOn("fp-collectModelInfo")) {
-    for (std::set<Node>::const_iterator i(relevantTerms.begin());
-         i != relevantTerms.end(); ++i) {
-      Trace("fp-collectModelInfo")
-          << "TheoryFp::collectModelInfo(): relevantTerms " << *i << std::endl;
+  Trace("fp-collectModelValues")
+      << "TheoryFp::collectModelValues(): begin" << std::endl;
+  if (TraceIsOn("fp-collectModelValues"))
+  {
+    for (std::set<Node>::const_iterator i(termSet.begin());
+         i != termSet.end();
+         ++i)
+    {
+      Trace("fp-collectModelValues")
+          << "TheoryFp::collectModelValues(): termSet " << *i
+          << std::endl;
     }
   }
-  for (const Node& node : relevantTerms)
+  for (const Node& node : termSet)
   {
     TypeNode t = node.getType();
     if ((!t.isRoundingMode() && !t.isFloatingPoint()) || !this->isLeaf(node))
@@ -796,8 +910,8 @@ bool TheoryFp::collectModelValues(TheoryModel* m,
       continue;
     }
 
-    Trace("fp-collectModelInfo")
-        << "TheoryFp::collectModelInfo(): " << node << std::endl;
+    Trace("fp-collectModelValues")
+        << "TheoryFp::collectModelValues(): " << node << std::endl;
 
     Node wordBlasted = d_wordBlaster->getValue(d_valuation, node);
     // We only assign the value if the FpWordBlaster actually has one, that is,
@@ -805,7 +919,7 @@ bool TheoryFp::collectModelValues(TheoryModel* m,
     if (!wordBlasted.isNull() && !m->assertEquality(node, wordBlasted, true))
     {
       Trace("fp-collectModelInfo")
-          << "TheoryFp::collectModelInfo(): ... not converted" << std::endl;
+          << "TheoryFp::collectModelValues(): ... not converted" << std::endl;
       return false;
     }
 
