@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Andrew Reynolds, Gereon Kremer, Mathias Preiner
+ *   Andrew Reynolds, Mathias Preiner, Gereon Kremer
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -41,157 +41,18 @@ namespace cvc5::internal {
 namespace theory {
 namespace quantifiers {
 
-CegTermType mkStrictCTT(CegTermType c)
-{
-  Assert(!isStrictCTT(c));
-  if (c == CEG_TT_LOWER)
-  {
-    return CEG_TT_LOWER_STRICT;
-  }
-  else if (c == CEG_TT_UPPER)
-  {
-    return CEG_TT_UPPER_STRICT;
-  }
-  return c;
-}
-
-CegTermType mkNegateCTT(CegTermType c)
-{
-  if (c == CEG_TT_LOWER)
-  {
-    return CEG_TT_UPPER;
-  }
-  else if (c == CEG_TT_UPPER)
-  {
-    return CEG_TT_LOWER;
-  }
-  else if (c == CEG_TT_LOWER_STRICT)
-  {
-    return CEG_TT_UPPER_STRICT;
-  }
-  else if (c == CEG_TT_UPPER_STRICT)
-  {
-    return CEG_TT_LOWER_STRICT;
-  }
-  return c;
-}
-bool isStrictCTT(CegTermType c)
-{
-  return c == CEG_TT_LOWER_STRICT || c == CEG_TT_UPPER_STRICT;
-}
-bool isLowerBoundCTT(CegTermType c)
-{
-  return c == CEG_TT_LOWER || c == CEG_TT_LOWER_STRICT;
-}
-bool isUpperBoundCTT(CegTermType c)
-{
-  return c == CEG_TT_UPPER || c == CEG_TT_UPPER_STRICT;
-}
-
-std::ostream& operator<<(std::ostream& os, CegInstEffort e)
-{
-  switch (e)
-  {
-    case CEG_INST_EFFORT_NONE: os << "?"; break;
-    case CEG_INST_EFFORT_STANDARD: os << "STANDARD"; break;
-    case CEG_INST_EFFORT_STANDARD_MV: os << "STANDARD_MV"; break;
-    case CEG_INST_EFFORT_FULL: os << "FULL"; break;
-    default: Unreachable();
-  }
-  return os;
-}
-
-std::ostream& operator<<(std::ostream& os, CegInstPhase phase)
-{
-  switch (phase)
-  {
-    case CEG_INST_PHASE_NONE: os << "?"; break;
-    case CEG_INST_PHASE_EQC: os << "eqc"; break;
-    case CEG_INST_PHASE_EQUAL: os << "eq"; break;
-    case CEG_INST_PHASE_ASSERTION: os << "as"; break;
-    case CEG_INST_PHASE_MVALUE: os << "mv"; break;
-    default: Unreachable();
-  }
-  return os;
-}
-std::ostream& operator<<(std::ostream& os, CegHandledStatus status)
-{
-  switch (status)
-  {
-    case CEG_UNHANDLED: os << "unhandled"; break;
-    case CEG_PARTIALLY_HANDLED: os << "partially_handled"; break;
-    case CEG_HANDLED: os << "handled"; break;
-    case CEG_HANDLED_UNCONDITIONAL: os << "handled_unc"; break;
-    default: Unreachable();
-  }
-  return os;
-}
-
-void TermProperties::composeProperty(TermProperties& p)
-{
-  if (p.d_coeff.isNull())
-  {
-    return;
-  }
-  if (d_coeff.isNull())
-  {
-    d_coeff = p.d_coeff;
-  }
-  else
-  {
-    d_coeff = arith::multConstants(d_coeff, p.d_coeff);
-  }
-}
-
-// push the substitution pv_prop.getModifiedTerm(pv) -> n
-void SolvedForm::push_back(Node pv, Node n, TermProperties& pv_prop)
-{
-  Assert(n.getType() == pv.getType());
-  d_vars.push_back(pv);
-  d_subs.push_back(n);
-  d_props.push_back(pv_prop);
-  if (pv_prop.isBasic())
-  {
-    return;
-  }
-  d_non_basic.push_back(pv);
-  // update theta value
-  Node new_theta = getTheta();
-  if (new_theta.isNull())
-  {
-    new_theta = pv_prop.d_coeff;
-  }
-  else
-  {
-    new_theta = arith::multConstants(new_theta, pv_prop.d_coeff);
-  }
-  d_theta.push_back(new_theta);
-}
-// pop the substitution pv_prop.getModifiedTerm(pv) -> n
-void SolvedForm::pop_back(Node pv, Node n, TermProperties& pv_prop)
-{
-  d_vars.pop_back();
-  d_subs.pop_back();
-  d_props.pop_back();
-  if (pv_prop.isBasic())
-  {
-    return;
-  }
-  d_non_basic.pop_back();
-  // update theta value
-  d_theta.pop_back();
-}
-
 CegInstantiator::CegInstantiator(Env& env,
                                  Node q,
                                  QuantifiersState& qs,
-                                 TermRegistry& tr,
-                                 InstStrategyCegqi* parent)
+                                 QuantifiersInferenceManager& qim,
+                                 QuantifiersRegistry& qr,
+                                 TermRegistry& tr)
     : EnvObj(env),
       d_quant(q),
       d_qstate(qs),
+      d_qim(qim),
+      d_qreg(qr),
       d_treg(tr),
-      d_parent(parent),
       d_is_nested_quant(false),
       d_effort(CEG_INST_EFFORT_NONE)
 {
@@ -487,7 +348,8 @@ void CegInstantiator::activateInstantiationVariable(Node v, unsigned index)
     Instantiator * vinst;
     if (tn.isRealOrInt())
     {
-      vinst = new ArithInstantiator(d_env, tn, d_parent->getVtsTermCache());
+      VtsTermCache* vtc = d_treg.getVtsTermCache();
+      vinst = new ArithInstantiator(d_env, tn, vtc);
     }
     else if (tn.isDatatype())
     {
@@ -495,7 +357,7 @@ void CegInstantiator::activateInstantiationVariable(Node v, unsigned index)
     }
     else if (tn.isBitVector())
     {
-      vinst = new BvInstantiator(d_env, tn, d_parent->getBvInverter());
+      vinst = new BvInstantiator(d_env, tn, d_treg.getBvInverter());
     }
     else if (tn.isBoolean())
     {
@@ -1108,13 +970,36 @@ bool CegInstantiator::doAddInstantiation(std::vector<Node>& vars,
     }
   }
   Trace("cegqi-inst-debug") << "Do the instantiation...." << std::endl;
-  return d_parent->doAddInstantiation(subs);
+
+  Assert(!d_quant.isNull());
+  // check if we need virtual term substitution (if used delta or infinity)
+  VtsTermCache* vtc = d_treg.getVtsTermCache();
+  bool usedVts = vtc->containsVtsTerm(subs, false);
+  Instantiate* inst = d_qim.getInstantiate();
+  // if doing partial quantifier elimination, record the instantiation and set
+  // the incomplete flag instead of sending instantiation lemma
+  if (d_qreg.getQuantAttributes().isQuantElimPartial(d_quant))
+  {
+    inst->recordInstantiation(d_quant, subs, usedVts);
+    return true;
+  }
+  else if (inst->addInstantiation(d_quant,
+                                  subs,
+                                  InferenceId::QUANTIFIERS_INST_CEGQI,
+                                  Node::null(),
+                                  usedVts))
+  {
+    return true;
+  }
+  // this should never happen for monotonic selection strategies
+  Trace("cegqi-warn") << "WARNING: Existing instantiation" << std::endl;
+  return false;
 }
 
 bool CegInstantiator::isEligibleForInstantiation(Node n) const
 {
   Kind nk = n.getKind();
-  if (nk != INST_CONSTANT && nk != SKOLEM && nk != BOOLEAN_TERM_VARIABLE)
+  if (nk != INST_CONSTANT && nk != SKOLEM)
   {
     return true;
   }
@@ -1513,22 +1398,35 @@ void CegInstantiator::markSolved(Node n, bool solved)
   }
 }
 
-void CegInstantiator::collectCeAtoms( Node n, std::map< Node, bool >& visited ) {
-  if( n.getKind()==FORALL ){
-    d_is_nested_quant = true;
-  }else if( visited.find( n )==visited.end() ){
-    visited[n] = true;
-    if( TermUtil::isBoolConnectiveTerm( n ) ){
-      for( unsigned i=0; i<n.getNumChildren(); i++ ){
-        collectCeAtoms( n[i], visited );
+void CegInstantiator::collectCeAtoms(Node n)
+{
+  std::unordered_set<TNode> visited;
+  std::vector<TNode> visit;
+  TNode cur;
+  visit.push_back(n);
+  do
+  {
+    cur = visit.back();
+    visit.pop_back();
+    if (visited.find(cur) == visited.end())
+    {
+      visited.insert(cur);
+      if (cur.getKind() == FORALL)
+      {
+        d_is_nested_quant = true;
       }
-    }else{
-      if( std::find( d_ce_atoms.begin(), d_ce_atoms.end(), n )==d_ce_atoms.end() ){
-        Trace("cegqi-ce-atoms") << "CE atoms : " << n << std::endl;
-        d_ce_atoms.push_back( n );
+      if (TermUtil::isBoolConnectiveTerm(cur))
+      {
+        visit.insert(visit.end(), cur.begin(), cur.end());
+      }
+      else if (std::find(d_ce_atoms.begin(), d_ce_atoms.end(), cur)
+               == d_ce_atoms.end())
+      {
+        Trace("cegqi-ce-atoms") << "CE atoms : " << cur << std::endl;
+        d_ce_atoms.push_back(cur);
       }
     }
-  }
+  } while (!visit.empty());
 }
 
 void CegInstantiator::registerCounterexampleLemma(Node lem,
@@ -1653,30 +1551,15 @@ void CegInstantiator::registerCounterexampleLemma(Node lem,
   // collect atoms from all lemmas: we will only solve for literals coming from
   // the original body
   d_is_nested_quant = false;
-  std::map< Node, bool > visited;
-  collectCeAtoms(lem, visited);
+  Node lemr = rewrite(lem);
+  collectCeAtoms(lemr);
   for (const Node& alem : auxLems)
   {
-    collectCeAtoms(alem, visited);
+    Node alemr = rewrite(alem);
+    collectCeAtoms(alemr);
   }
 }
 
-Instantiator::Instantiator(Env& env, TypeNode tn) : EnvObj(env), d_type(tn)
-{
-  d_closed_enum_type = tn.isClosedEnumerable();
-}
-
-bool Instantiator::processEqualTerm(CegInstantiator* ci,
-                                    SolvedForm& sf,
-                                    Node pv,
-                                    TermProperties& pv_prop,
-                                    Node n,
-                                    CegInstEffort effort)
-{
-  pv_prop.d_type = CEG_TT_EQUAL;
-  return ci->constructInstantiationInc(pv, n, pv_prop, sf);
-}
-
 }  // namespace quantifiers
 }  // namespace theory
 }  // namespace cvc5::internal
