@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -56,18 +56,12 @@ InstStrategyCegqi::InstStrategyCegqi(Env& env,
       d_cbqi_set_quant_inactive(false),
       d_incomplete_check(false),
       d_added_cbqi_lemma(userContext()),
-      d_bv_invert(nullptr),
       d_small_const_multiplier(NodeManager::currentNM()->mkConstReal(
           Rational(1) / Rational(1000000))),
       d_small_const(d_small_const_multiplier),
-      d_freeDeltaLb(false, userContext())
+      d_freeDeltaLb(userContext(), false)
 {
   d_check_vts_lemma_lc = false;
-  if (options().quantifiers.cegqiBv)
-  {
-    // if doing instantiation for BV, need the inverter class
-    d_bv_invert.reset(new BvInverter(env.getOptions(), env.getRewriter()));
-  }
   if (options().quantifiers.cegqiNestedQE)
   {
     d_nestedQe.reset(new NestedQe(d_env));
@@ -272,6 +266,11 @@ void InstStrategyCegqi::check(Theory::Effort e, QEffort quant_e)
       for( std::map< Node, bool >::iterator it = d_active_quant.begin(); it != d_active_quant.end(); ++it ){
         Node q = it->first;
         Trace("cegqi") << "CBQI : Process quantifier " << q[0] << " at effort " << ee << std::endl;
+        if (d_qreg.getQuantAttributes().isQuantElimPartial(q))
+        {
+          d_cbqi_set_quant_inactive = true;
+          d_incomplete_check = true;
+        }
         process(q, e, ee);
         if (d_qstate.isInConflict())
         {
@@ -504,53 +503,17 @@ Node InstStrategyCegqi::getCounterexampleLiteral(Node q)
   return ceLit;
 }
 
-bool InstStrategyCegqi::doAddInstantiation( std::vector< Node >& subs ) {
-  Assert(!d_curr_quant.isNull());
-  // check if we need virtual term substitution (if used delta or infinity)
-  VtsTermCache* vtc = d_treg.getVtsTermCache();
-  bool usedVts = vtc->containsVtsTerm(subs, false);
-  Instantiate* inst = d_qim.getInstantiate();
-  //if doing partial quantifier elimination, record the instantiation and set the incomplete flag instead of sending instantiation lemma
-  if (d_qreg.getQuantAttributes().isQuantElimPartial(d_curr_quant))
-  {
-    d_cbqi_set_quant_inactive = true;
-    d_incomplete_check = true;
-    inst->recordInstantiation(d_curr_quant, subs, usedVts);
-    return true;
-  }
-  else if (inst->addInstantiation(d_curr_quant,
-                                  subs,
-                                  InferenceId::QUANTIFIERS_INST_CEGQI,
-                                  Node::null(),
-                                  usedVts))
-  {
-    return true;
-  }
-  // this should never happen for monotonic selection strategies
-  Trace("cegqi-warn") << "WARNING: Existing instantiation" << std::endl;
-  return false;
-}
-
 CegInstantiator * InstStrategyCegqi::getInstantiator( Node q ) {
   std::map<Node, std::unique_ptr<CegInstantiator>>::iterator it =
       d_cinst.find(q);
   if( it==d_cinst.end() ){
-    d_cinst[q].reset(new CegInstantiator(d_env, q, d_qstate, d_treg, this));
+    d_cinst[q].reset(
+        new CegInstantiator(d_env, q, d_qstate, d_qim, d_qreg, d_treg));
     return d_cinst[q].get();
   }
   return it->second.get();
 }
 
-VtsTermCache* InstStrategyCegqi::getVtsTermCache() const
-{
-  return d_treg.getVtsTermCache();
-}
-
-BvInverter* InstStrategyCegqi::getBvInverter() const
-{
-  return d_bv_invert.get();
-}
-
 bool InstStrategyCegqi::processNestedQe(Node q, bool isPreregister)
 {
   if (d_nestedQe != nullptr)
