@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -15,6 +15,7 @@
 
 #include "theory/quantifiers/ieval/term_evaluator.h"
 
+#include "expr/node_algorithm.h"
 #include "theory/quantifiers/ieval/state.h"
 #include "theory/quantifiers/quantifiers_state.h"
 #include "theory/quantifiers/term_database.h"
@@ -44,6 +45,10 @@ TermEvaluatorEntailed::TermEvaluatorEntailed(Env& env,
 
 TNode TermEvaluatorEntailed::evaluateBase(const State& s, TNode n)
 {
+  if (n.getKind() == FORALL)
+  {
+    return s.getSome();
+  }
   // if unknown, it is none
   return d_qs.hasTerm(n) ? d_qs.getRepresentative(n) : s.getNone();
 }
@@ -53,8 +58,9 @@ TNode TermEvaluatorEntailed::partialEvaluateChild(
 {
   // if a Boolean connective, handle short circuiting
   Kind k = n.getKind();
-  // implies and xor are eliminated from quantifier bodies
-  Assert(k != IMPLIES && k != XOR);
+  // Implies and xor are eliminated from the propositional skeleton of
+  // quantifier bodies, so we don't check for them here. They still may
+  // occur e.g. as arguments to parameteric operators involving Bool.
   if (k == AND || k == OR)
   {
     if (val.isConst() && val.getConst<bool>() == (k == OR))
@@ -147,7 +153,11 @@ TNode TermEvaluatorEntailed::evaluate(const State& s,
 {
   // set to unknown, handle cases
   TNode ret = s.getNone();
-
+  // if an existing ground term, just return representative
+  if (!expr::hasBoundVar(n) && d_qs.hasTerm(n))
+  {
+    return d_qs.getRepresentative(n);
+  }
   TNode mop = d_tdb.getMatchOperator(n);
   if (!mop.isNull())
   {
