@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -201,6 +201,13 @@ bool CegisCoreConnective::processInitialize(Node conj,
       d_qim.lemma(tst.negate(),
                   InferenceId::QUANTIFIERS_SYGUS_CEGIS_UCL_SYM_BREAK);
     }
+    else
+    {
+      Trace("sygus-ccore-init") << "  will use " << (r == 0 ? "pre" : "post")
+                                << "condition as a filter." << std::endl;
+      // just use as a filtering
+      c.initialize(node, Node::null());
+    }
   }
   if (!isActive())
   {
@@ -293,6 +300,7 @@ bool CegisCoreConnective::constructSolution(
     Node fpred = cfilter.getFormula();
     if (!fpred.isNull() && !fpred.isConst())
     {
+      Trace("sygus-ccore-debug") << "...check filter pred " << fpred << std::endl;
       // check refinement points
       Node etsrn = d == 0 ? etsr : etsr.negate();
       std::unordered_set<Node> visited;
@@ -300,6 +308,7 @@ bool CegisCoreConnective::constructSolution(
       Node rid = cfilter.getRefinementPt(this, etsrn, visited, pt);
       if (!rid.isNull())
       {
+        Trace("sygus-ccore-debug") << "...failed refinement" << std::endl;
         // failed a refinement point
         continue;
       }
@@ -398,9 +407,10 @@ void CegisCoreConnective::Component::addRefinementPt(
     Node id, const std::vector<Node>& pt)
 {
   d_numRefPoints++;
-  bool res = d_refinementPt.addTerm(id, pt);
-  // this should always be a new point
-  AlwaysAssert(res);
+  d_refinementPt.addTerm(id, pt);
+  // Note that the above method returns false if pt is a duplicate of
+  // a previous point. This may happen if the candidate solutions we are testing
+  // involve partial functions.
 }
 void CegisCoreConnective::Component::addFalseCore(Node id,
                                                   const std::vector<Node>& u)
@@ -674,7 +684,10 @@ Node CegisCoreConnective::constructSolutionFromPool(Component& ccheck,
     checkSol->setOption("produce-unsat-cores", "true");
     Trace("sygus-ccore") << "----- Check candidate " << an << std::endl;
     std::vector<Node> rasserts = asserts;
-    rasserts.push_back(d_sc);
+    if (!d_sc.isNull())
+    {
+      rasserts.push_back(d_sc);
+    }
     rasserts.push_back(ccheck.getFormula());
     std::shuffle(rasserts.begin(), rasserts.end(), Random::getRandom());
     Node query = rasserts.size() == 1 ? rasserts[0] : nm->mkNode(AND, rasserts);
@@ -694,7 +707,10 @@ Node CegisCoreConnective::constructSolutionFromPool(Component& ccheck,
       std::vector<Node> uasserts;
       std::unordered_set<Node> queryAsserts;
       queryAsserts.insert(ccheck.getFormula());
-      queryAsserts.insert(d_sc);
+      if (!d_sc.isNull())
+      {
+        queryAsserts.insert(d_sc);
+      }
       bool hasQuery =
           getUnsatCoreFromSubsolver(*checkSol, queryAsserts, uasserts);
       // now, check the side condition
@@ -744,16 +760,31 @@ Node CegisCoreConnective::constructSolutionFromPool(Component& ccheck,
       {
         // In terms of Variant #2, this is the line:
         //   "return u_1 AND ... AND u_m where U = { u_1, ..., u_m }".
-        Trace("sygus-ccore") << ">>> Solution : " << uasserts << std::endl;
         // We convert the builtin solution to a sygus datatype to
         // communicate with the sygus solver.
+        if (uasserts.empty())
+        {
+          // In the rare case in which the side condition implies the goal
+          // already, then uasserts may be empty and any solution suffices.
+          // Take the last enumerated term from the pool.
+          Assert (!passerts.empty());
+          uasserts.push_back(passerts.back());
+        }
+        Trace("sygus-ccore") << ">>> Solution : " << uasserts << std::endl;
         Node sol = ccheck.getSygusSolution(uasserts);
         Trace("sygus-ccore-sy") << "Sygus solution : " << sol << std::endl;
         return sol;
       }
+      else if (uasserts.empty())
+      {
+        // should never happen, since we check that side condition is
+        // satisfiable when initializing the sygus conjecture
+        Assert(false);
+        Trace("sygus-ccore") << "--- Empty core, skip" << std::endl;
+        return Node::null();
+      }
       else
       {
-        Assert(!uasserts.empty());
         Node xu = uasserts[0];
         Trace("sygus-ccore")
             << "--- Add false core : " << uasserts << std::endl;
