@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Andrew Reynolds, Gereon Kremer, Andres Noetzli
+ *   Andrew Reynolds, Gereon Kremer, Abdalrhman Mohamed
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -27,9 +27,9 @@
 #include "theory/quantifiers/first_order_model.h"
 #include "theory/quantifiers/instantiate.h"
 #include "theory/quantifiers/quantifiers_attributes.h"
+#include "theory/quantifiers/sygus/embedding_converter.h"
 #include "theory/quantifiers/sygus/enum_value_manager.h"
 #include "theory/quantifiers/sygus/print_sygus_to_builtin.h"
-#include "theory/quantifiers/sygus/sygus_grammar_cons.h"
 #include "theory/quantifiers/sygus/sygus_pbe.h"
 #include "theory/quantifiers/sygus/synth_engine.h"
 #include "theory/quantifiers/sygus/term_database_sygus.h"
@@ -60,15 +60,12 @@ SynthConjecture::SynthConjecture(Env& env,
       d_verify(env, d_tds),
       d_hasSolution(false),
       d_computedSolution(false),
-      d_runExprMiner(options().quantifiers.sygusRewSynth
-                     || options().quantifiers.sygusQueryGen
-                            != options::SygusQueryGenMode::NONE
-                     || options().quantifiers.sygusFilterSolMode
-                            != options::SygusFilterSolMode::NONE),
+      d_runExprMiner(options().quantifiers.sygusFilterSolMode
+                     != options::SygusFilterSolMode::NONE),
       d_ceg_si(new CegSingleInv(env, tr, s)),
       d_templInfer(new SygusTemplateInfer(env)),
       d_ceg_proc(new SynthConjectureProcess(env)),
-      d_ceg_gc(new CegGrammarConstructor(env, d_tds, this)),
+      d_embConv(new EmbeddingConverter(env, d_tds, this)),
       d_sygus_rconst(new SygusRepairConst(env, d_tds)),
       d_exampleInfer(new ExampleInfer(d_tds)),
       d_ceg_pbe(new SygusPbe(env, qs, qim, d_tds, this)),
@@ -107,7 +104,8 @@ void SynthConjecture::presolve()
   // a new, next solution in their enumeration.
   if (d_hasSolution)
   {
-    excludeCurrentSolution(d_solutionValues.back());
+    excludeCurrentSolution(d_solutionValues.back(),
+                           InferenceId::QUANTIFIERS_SYGUS_INC_EXCLUDE_CURRENT);
     // we don't have a solution yet
     d_hasSolution = false;
     d_computedSolution = false;
@@ -132,10 +130,15 @@ void SynthConjecture::assign(Node q)
   QAttributes qa;
   QuantAttributes::computeQuantAttributes(q, qa);
 
+  Node sc = qa.d_sygusSideCondition;
+  // we check whether the conjecture is single invocation if we are marked
+  // with the sygus attribute and don't have a side condition
+  bool checkSingleInvocation = qa.d_sygus && sc.isNull();
+
   std::map<Node, Node> templates;
   std::map<Node, Node> templates_arg;
   // register with single invocation if applicable
-  if (qa.d_sygus)
+  if (checkSingleInvocation)
   {
     d_ceg_si->initialize(d_simp_quant);
     d_simp_quant = d_ceg_si->getSimplifiedConjecture();
@@ -161,32 +164,68 @@ void SynthConjecture::assign(Node q)
   // finished simplifying the quantified formula at this point
 
   // convert to deep embedding and finalize single invocation here
-  d_embed_quant = d_ceg_gc->process(d_simp_quant, templates, templates_arg);
+  d_embed_quant = d_embConv->process(d_simp_quant, templates, templates_arg);
   Trace("cegqi") << "SynthConjecture : converted to embedding : "
                  << d_embed_quant << std::endl;
 
-  Node sc = qa.d_sygusSideCondition;
   if (!sc.isNull())
   {
+    Trace("cegqi-debug") << "Side condition is: " << sc << std::endl;
+    // Immediately check if unsat, use lambda returning true for functions
+    // to synthesize.
+    std::vector<Node> vars;
+    std::vector<Node> subs;
+    for (const Node& v : q[0])
+    {
+      vars.push_back(v);
+      TypeNode vtype = v.getType();
+      Assert(vtype.isBoolean()
+             || (vtype.isFunction() && vtype.getRangeType().isBoolean()));
+      Node s = nm->mkConst(true);
+      if (vtype.isFunction())
+      {
+        std::vector<TypeNode> atypes = vtype.getArgTypes();
+        std::vector<Node> lvars;
+        for (const TypeNode& tn : atypes)
+        {
+          lvars.push_back(nm->mkBoundVar(tn));
+        }
+        s = nm->mkNode(LAMBDA, nm->mkNode(BOUND_VAR_LIST, lvars), s);
+      }
+      subs.push_back(s);
+    }
+    Node ksc =
+        sc.substitute(vars.begin(), vars.end(), subs.begin(), subs.end());
+    Result r = d_verify.verify(ksc);
+    // if infeasible, we are done
+    if (r.getStatus() == Result::UNSAT)
+    {
+      d_qim.lemma(d_quant.negate(),
+                  InferenceId::QUANTIFIERS_SYGUS_SC_INFEASIBLE);
+      return;
+    }
     // convert to deep embedding
-    d_embedSideCondition = d_ceg_gc->convertToEmbedding(sc);
+    d_embedSideCondition = d_embConv->convertToEmbedding(sc);
     Trace("cegqi") << "SynthConjecture : side condition : "
                    << d_embedSideCondition << std::endl;
   }
 
   // we now finalize the single invocation module, based on the syntax
   // restrictions
-  if (qa.d_sygus)
+  if (checkSingleInvocation)
   {
-    d_ceg_si->finishInit(d_ceg_gc->isSyntaxRestricted());
+    d_ceg_si->finishInit(d_embConv->isSyntaxRestricted());
   }
 
   Assert(d_candidates.empty());
   std::vector<Node> vars;
-  for (unsigned i = 0; i < d_embed_quant[0].getNumChildren(); i++)
+  for (size_t i = 0, nvars = d_embed_quant[0].getNumChildren(); i < nvars; i++)
   {
-    vars.push_back(d_embed_quant[0][i]);
-    Node e = sm->mkDummySkolem("e", d_embed_quant[0][i].getType());
+    Node v = d_embed_quant[0][i];
+    vars.push_back(v);
+    Node e = sm->mkSkolemFunction(SkolemFunId::QUANTIFIERS_SYNTH_FUN_EMBED,
+                                  v.getType(),
+                                  d_simp_quant[0][i]);
     d_candidates.push_back(e);
   }
   Trace("cegqi") << "Base quantified formula is : " << d_embed_quant
@@ -194,7 +233,7 @@ void SynthConjecture::assign(Node q)
   // construct base instantiation
   Subs bsubs;
   bsubs.add(vars, d_candidates);
-  d_base_inst = rewrite(bsubs.apply(d_embed_quant[1]));
+  d_base_inst = d_tds->rewriteNode(bsubs.apply(d_embed_quant[1]));
   d_checkBody = d_embed_quant[1];
   if (d_checkBody.getKind() == NOT && d_checkBody[0].getKind() == FORALL)
   {
@@ -207,7 +246,8 @@ void SynthConjecture::assign(Node q)
     }
     d_checkBody = d_checkBody[0][1].negate();
   }
-  d_checkBody = rewrite(bsubs.apply(d_checkBody));
+  d_checkBody = bsubs.apply(d_checkBody);
+  d_checkBody = d_tds->rewriteNode(d_checkBody);
   if (!d_embedSideCondition.isNull() && !vars.empty())
   {
     d_embedSideCondition = d_embedSideCondition.substitute(
@@ -413,7 +453,7 @@ bool SynthConjecture::doCheck()
             if (TraceIsOn("sygus-engine-rr"))
             {
               Node bv = d_tds->sygusToBuiltin(nv, tn);
-              bv = rewrite(bv);
+              bv = d_tds->rewriteNode(bv);
               Trace("sygus-engine-rr") << " -> " << bv << std::endl;
             }
           }
@@ -443,12 +483,13 @@ bool SynthConjecture::doCheck()
     }
   }
 
-  // check the side condition if we constructed a candidate
   if (constructed_cand)
   {
+    // check the side condition if we constructed a candidate
     if (!checkSideCondition(candidate_values))
     {
-      excludeCurrentSolution(candidate_values);
+      excludeCurrentSolution(candidate_values,
+                             InferenceId::QUANTIFIERS_SYGUS_SC_EXCLUDE_CURRENT);
       Trace("sygus-engine") << "...failed side condition" << std::endl;
       return false;
     }
@@ -536,7 +577,9 @@ bool SynthConjecture::doCheck()
     // In the rare case that the subcall is unknown, we simply exclude the
     // solution, without adding a counterexample point. This should only
     // happen if the quantifier free logic is undecidable.
-    excludeCurrentSolution(candidate_values);
+    excludeCurrentSolution(
+        candidate_values,
+        InferenceId::QUANTIFIERS_SYGUS_NO_VERIFY_EXCLUDE_CURRENT);
     // We should set refutation unsound, since an "unsat" answer should not be
     // interpreted as "infeasible", which would make a difference in the rare
     // case where e.g. we had a finite grammar and exhausted the grammar.
@@ -555,7 +598,9 @@ bool SynthConjecture::doCheck()
   if (runExprMiner())
   {
     // excluded due to expression mining
-    excludeCurrentSolution(candidate_values);
+    excludeCurrentSolution(
+        candidate_values,
+        InferenceId::QUANTIFIERS_SYGUS_STREAM_EXCLUDE_CURRENT);
     // streaming means now we immediately are looking for a new solution
     d_hasSolution = false;
     d_computedSolution = false;
@@ -626,7 +671,8 @@ bool SynthConjecture::processCounterexample(const std::vector<Node>& skModel)
                                 << std::endl;
     std::vector<Node> cvals = d_solutionValues.back();
     // something went wrong, exclude the current candidate
-    excludeCurrentSolution(cvals);
+    excludeCurrentSolution(
+        cvals, InferenceId::QUANTIFIERS_SYGUS_REPEAT_CEX_EXCLUDE_CURRENT);
     // Note this happens when evaluation is incapable of disproving a candidate
     // for counterexample point c, but satisfiability checking happened to find
     // the the same point c is indeed a true counterexample. It is sound
@@ -718,8 +764,7 @@ ExpressionMinerManager* SynthConjecture::getExprMinerManagerFor(Node e)
   }
   d_exprm[e].reset(new ExpressionMinerManager(d_env));
   ExpressionMinerManager* emm = d_exprm[e].get();
-  emm->initializeSygus(d_tds, e, options().quantifiers.sygusSamples, true);
-  emm->initializeMinersForOptions();
+  emm->initializeSygus(e.getType());
   return emm;
 }
 
@@ -736,7 +781,8 @@ void SynthConjecture::debugPrint(const char* c)
   Trace(c) << "  * Counterexample skolems : " << d_innerSks << std::endl;
 }
 
-void SynthConjecture::excludeCurrentSolution(const std::vector<Node>& values)
+void SynthConjecture::excludeCurrentSolution(const std::vector<Node>& values,
+                                             InferenceId id)
 {
   Assert(values.size() == d_candidates.size());
   Trace("cegqi-debug") << "Exclude current solution: " << values << std::endl;
@@ -761,9 +807,9 @@ void SynthConjecture::excludeCurrentSolution(const std::vector<Node>& values)
                        ? exp[0]
                        : NodeManager::currentNM()->mkNode(kind::AND, exp);
     exc_lem = exc_lem.negate();
-    Trace("cegqi-lemma") << "Cegqi::Lemma : stream exclude current solution : "
-                         << exc_lem << std::endl;
-    d_qim.lemma(exc_lem, InferenceId::QUANTIFIERS_SYGUS_STREAM_EXCLUDE_CURRENT);
+    Trace("cegqi-lemma") << "Cegqi::Lemma : exclude current solution : "
+                         << exc_lem << " by " << id << std::endl;
+    d_qim.lemma(exc_lem, id);
   }
 }
 
@@ -802,13 +848,7 @@ bool SynthConjecture::runExprMiner()
       ExpressionMinerManager* emm = getExprMinerManagerFor(e);
       if (emm != nullptr)
       {
-        bool rew_print = false;
-        bool ret = emm->addTerm(sol, out, rew_print);
-        if (rew_print)
-        {
-          // count the number of rewrites we printed
-          ++(d_stats.d_candidate_rewrites_print);
-        }
+        bool ret = emm->addTerm(sol);
         if (!ret)
         {
           // count the number of filtered solutions
