@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -25,7 +25,7 @@
 #include "theory/datatypes/sygus_datatype_utils.h"
 #include "theory/quantifiers/first_order_model.h"
 #include "theory/quantifiers/sygus/sygus_enumerator.h"
-#include "theory/quantifiers/sygus/sygus_grammar_cons.h"
+#include "theory/quantifiers/sygus/sygus_grammar_cons_new.h"
 #include "theory/quantifiers/sygus/synth_engine.h"
 #include "theory/quantifiers/term_util.h"
 #include "theory/rewriter.h"
@@ -172,15 +172,14 @@ void getMinGroundTerms(const Options& options,
  * @param extra_cons: A map of TypeNode to constants, which are added in
  *                    addition to the default grammar.
  */
-void addSpecialValues(const TypeNode& tn,
-                      std::map<TypeNode, std::unordered_set<Node>>& extra_cons)
+void addSpecialValues(const TypeNode& tn, std::vector<Node>& extra_cons)
 {
   if (tn.isBitVector())
   {
     uint32_t size = tn.getBitVectorSize();
-    extra_cons[tn].insert(bv::utils::mkOnes(size));
-    extra_cons[tn].insert(bv::utils::mkMinSigned(size));
-    extra_cons[tn].insert(bv::utils::mkMaxSigned(size));
+    extra_cons.push_back(bv::utils::mkOnes(size));
+    extra_cons.push_back(bv::utils::mkMinSigned(size));
+    extra_cons.push_back(bv::utils::mkMaxSigned(size));
   }
 }
 
@@ -365,10 +364,7 @@ void SygusInst::registerQuantifier(Node q)
 
   Trace("sygus-inst") << "Register " << q << std::endl;
 
-  std::map<TypeNode, std::unordered_set<Node>> extra_cons;
-  std::map<TypeNode, std::unordered_set<Node>> exclude_cons;
-  std::map<TypeNode, std::unordered_set<Node>> include_cons;
-  std::unordered_set<Node> term_irrelevant;
+  std::vector<Node> extra_cons;
 
   /* Collect relevant local ground terms for each variable type. */
   if (options().quantifiers.sygusInstScope == options::SygusInstScope::IN
@@ -396,7 +392,7 @@ void SygusInst::registerQuantifier(Node q)
       for (const auto& t : terms)
       {
         TypeNode ttn = t.getType();
-        extra_cons[ttn].insert(t);
+        extra_cons.push_back(t);
         Trace("sygus-inst") << "Adding (local) extra cons: " << t << std::endl;
       }
     }
@@ -432,7 +428,7 @@ void SygusInst::registerQuantifier(Node q)
         for (const auto& t : (*it).second)
         {
           TypeNode ttn = t.getType();
-          extra_cons[ttn].insert(t);
+          extra_cons.push_back(t);
           Trace("sygus-inst")
               << "Adding (global) extra cons: " << t << std::endl;
         }
@@ -446,14 +442,8 @@ void SygusInst::registerQuantifier(Node q)
   for (const Node& var : q[0])
   {
     addSpecialValues(var.getType(), extra_cons);
-    TypeNode tn = CegGrammarConstructor::mkSygusDefaultType(options(),
-                                                            var.getType(),
-                                                            Node(),
-                                                            var.toString(),
-                                                            extra_cons,
-                                                            exclude_cons,
-                                                            include_cons,
-                                                            term_irrelevant);
+    TypeNode tn = SygusGrammarCons::mkDefaultSygusType(
+        options(), var.getType(), Node(), extra_cons);
     types.push_back(tn);
 
     Trace("sygus-inst") << "Construct (default) datatype for " << var
@@ -548,11 +538,7 @@ void SygusInst::registerCeLemma(Node q, std::vector<TypeNode>& types)
     // evaluation function, since we are not using the builtin support
     // for evaluation functions. We use the DT_SYGUS_EVAL term so that the
     // skolem construction here is deterministic and reproducible.
-    SkolemManager::SkolemFlags flags = eval.getType().isBoolean()
-                                           ? SkolemManager::SKOLEM_BOOL_TERM_VAR
-                                           : SkolemManager::SKOLEM_DEFAULT;
-    Node k = sm->mkPurifySkolem(
-        eval, "eval", "evaluation variable for sygus-inst", flags);
+    Node k = sm->mkPurifySkolem(eval);
     // Requires instantiation constant attribute as well. This ensures that
     // other instantiation methods, e.g. E-matching do not consider this term
     // for instantiation, as it is model-unsound to do so.
