@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -52,6 +52,11 @@ TermRegistry::TermRegistry(Env& env,
   {
     d_ochecker.reset(new OracleChecker(env));
   }
+  if (options().quantifiers.cegqiBv)
+  {
+    // if doing instantiation for BV, need the inverter class
+    d_bvInvert.reset(new BvInverter(options(), env.getRewriter()));
+  }
   if (options().quantifiers.sygus || options().quantifiers.sygusInst)
   {
     // must be constructed here since it is required for datatypes finistInit
@@ -182,6 +187,8 @@ TermPools* TermRegistry::getTermPools() const { return d_termPools.get(); }
 
 VtsTermCache* TermRegistry::getVtsTermCache() const { return d_vtsCache.get(); }
 
+BvInverter* TermRegistry::getBvInverter() const { return d_bvInvert.get(); }
+
 ieval::InstEvaluatorManager* TermRegistry::getInstEvaluatorManager() const
 {
   return d_ievalMan.get();
