@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Aina Niemetz, Andrew Reynolds, Morgan Deters
+ *   Andrew Reynolds, Aina Niemetz, Morgan Deters
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -21,6 +21,10 @@ namespace cvc5::internal {
 namespace theory {
 namespace quantifiers {
 
+TypeNode QuantifierTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode QuantifierTypeRule::computeType(NodeManager* nodeManager,
                                          TNode n,
                                          bool check,
@@ -70,6 +74,11 @@ TypeNode QuantifierTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode QuantifierBoundVarListTypeRule::preComputeType(NodeManager* nm,
+                                                        TNode n)
+{
+  return nm->boundVarListType();
+}
 TypeNode QuantifierBoundVarListTypeRule::computeType(NodeManager* nodeManager,
                                                      TNode n,
                                                      bool check,
@@ -90,6 +99,10 @@ TypeNode QuantifierBoundVarListTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->boundVarListType();
 }
 
+TypeNode QuantifierInstPatternTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->instPatternType();
+}
 TypeNode QuantifierInstPatternTypeRule::computeType(NodeManager* nodeManager,
                                                     TNode n,
                                                     bool check,
@@ -111,6 +124,10 @@ TypeNode QuantifierInstPatternTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->instPatternType();
 }
 
+TypeNode QuantifierAnnotationTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->instPatternType();
+}
 TypeNode QuantifierAnnotationTypeRule::computeType(NodeManager* nodeManager,
                                                    TNode n,
                                                    bool check,
@@ -160,6 +177,11 @@ TypeNode QuantifierAnnotationTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->instPatternType();
 }
 
+TypeNode QuantifierInstPatternListTypeRule::preComputeType(NodeManager* nm,
+                                                           TNode n)
+{
+  return nm->instPatternListType();
+}
 TypeNode QuantifierInstPatternListTypeRule::computeType(
     NodeManager* nodeManager, TNode n, bool check, std::ostream* errOut)
 {
@@ -182,6 +204,11 @@ TypeNode QuantifierInstPatternListTypeRule::computeType(
   }
   return nodeManager->instPatternListType();
 }
+TypeNode QuantifierOracleFormulaGenTypeRule::preComputeType(NodeManager* nm,
+                                                            TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode QuantifierOracleFormulaGenTypeRule::computeType(
     NodeManager* nodeManager, TNode n, bool check, std::ostream* errOut)
 {
