@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -154,7 +154,9 @@ Node Rewriter::rewriteTo(theory::TheoryId theoryId,
                          TConvProofGenerator* tcpg)
 {
 #ifdef CVC5_ASSERTIONS
-  bool isEquality = node.getKind() == kind::EQUAL && (!node[0].getType().isBoolean());
+  bool isEquality = node.getKind() == kind::EQUAL
+                    && !node[0].getType().isBoolean()
+                    && !node[1].getType().isBoolean();
 
   if (d_rewriteStack == nullptr)
   {
@@ -208,10 +210,13 @@ Node Rewriter::rewriteTo(theory::TheoryId theoryId,
 
           // Put the rewritten node to the top of the stack
           TNode newNode = response.d_node;
+          Trace("rewriter-debug") << "Pre-Rewrite: " << rewriteStackTop.d_node
+                                  << " to " << newNode << std::endl;
           TheoryId newTheory = theoryOf(newNode);
           rewriteStackTop.d_node = newNode;
           rewriteStackTop.d_theoryId = newTheory;
-          Assert(newNode.getType() == rewriteStackTop.d_node.getType())
+          Assert(newNode.getType().isComparableTo(
+              rewriteStackTop.d_node.getType()))
               << "Pre-rewriting " << rewriteStackTop.d_node << " to " << newNode
               << " does not preserve type";
           // In the pre-rewrite, if changing theories, we just call the other
@@ -297,11 +302,13 @@ Node Rewriter::rewriteTo(theory::TheoryId theoryId,
         Kind originalKind = rewriteStackTop.d_node.getKind();
         RewriteResponse response = postRewrite(
             rewriteStackTop.getTheoryId(), rewriteStackTop.d_node, tcpg);
-
         // We continue with the response we got
         TNode newNode = response.d_node;
+        Trace("rewriter-debug") << "Post-Rewrite: " << rewriteStackTop.d_node
+                                << " to " << newNode << std::endl;
         TheoryId newTheoryId = theoryOf(newNode);
-        Assert(newNode.getType() == rewriteStackTop.d_node.getType())
+        Assert(
+            newNode.getType().isComparableTo(rewriteStackTop.d_node.getType()))
             << "Post-rewriting " << rewriteStackTop.d_node << " to " << newNode
             << " does not preserve type";
         if (newTheoryId != rewriteStackTop.getTheoryId()
@@ -394,7 +401,7 @@ Node Rewriter::rewriteTo(theory::TheoryId theoryId,
     if (rewriteStack.size() == 1) {
       Assert(!isEquality || rewriteStackTop.d_node.getKind() == kind::EQUAL
              || rewriteStackTop.d_node.isConst());
-      Assert(rewriteStackTop.d_node.getType() == node.getType())
+      Assert(rewriteStackTop.d_node.getType().isComparableTo(node.getType()))
           << "Rewriting " << node << " to " << rewriteStackTop.d_node
           << " does not preserve type";
       return rewriteStackTop.d_node;
