@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -137,6 +137,8 @@ void SolverState::registerTerm(Node r, TypeNode tnn, Node n)
       else
       {
         d_congruent[n] = itb->second;
+        // consider it regardless of whether congruent
+        d_bop_index[nk][n[0]][n[1]] = n;
       }
     }
     d_nvar_sets[r].push_back(n);
