@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -30,6 +30,10 @@ namespace sets {
 
 using namespace cvc5::internal::theory::datatypes;
 
+TypeNode SetsBinaryOperatorTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode SetsBinaryOperatorTypeRule::computeType(NodeManager* nodeManager,
                                                  TNode n,
                                                  bool check,
@@ -68,6 +72,10 @@ bool SetsBinaryOperatorTypeRule::computeIsConst(NodeManager* nodeManager,
   return NormalForm::checkNormalConstant(n);
 }
 
+TypeNode SubsetTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode SubsetTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -91,6 +99,10 @@ TypeNode SubsetTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode MemberTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode MemberTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -121,6 +133,10 @@ TypeNode MemberTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode SingletonTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode SingletonTypeRule::computeType(NodeManager* nodeManager,
                                         TNode n,
                                         bool check,
@@ -137,6 +153,10 @@ bool SingletonTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
   return n[0].isConst();
 }
 
+TypeNode EmptySetTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode EmptySetTypeRule::computeType(NodeManager* nodeManager,
                                        TNode n,
                                        bool check,
@@ -147,6 +167,10 @@ TypeNode EmptySetTypeRule::computeType(NodeManager* nodeManager,
   return emptySet.getType();
 }
 
+TypeNode CardTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->integerType();
+}
 TypeNode CardTypeRule::computeType(NodeManager* nodeManager,
                                    TNode n,
                                    bool check,
@@ -165,6 +189,10 @@ TypeNode CardTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->integerType();
 }
 
+TypeNode ComplementTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ComplementTypeRule::computeType(NodeManager* nodeManager,
                                          TNode n,
                                          bool check,
@@ -183,6 +211,10 @@ TypeNode ComplementTypeRule::computeType(NodeManager* nodeManager,
   return setType;
 }
 
+TypeNode UniverseSetTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode UniverseSetTypeRule::computeType(NodeManager* nodeManager,
                                           TNode n,
                                           bool check,
@@ -201,6 +233,10 @@ TypeNode UniverseSetTypeRule::computeType(NodeManager* nodeManager,
   return setType;
 }
 
+TypeNode ComprehensionTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ComprehensionTypeRule::computeType(NodeManager* nodeManager,
                                             TNode n,
                                             bool check,
@@ -223,6 +259,10 @@ TypeNode ComprehensionTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->mkSetType(n[2].getType(check));
 }
 
+TypeNode ChooseTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ChooseTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -241,6 +281,10 @@ TypeNode ChooseTypeRule::computeType(NodeManager* nodeManager,
   return setType.getSetElementType();
 }
 
+TypeNode IsSingletonTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode IsSingletonTypeRule::computeType(NodeManager* nodeManager,
                                           TNode n,
                                           bool check,
@@ -259,6 +303,10 @@ TypeNode IsSingletonTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode InsertTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode InsertTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -289,6 +337,10 @@ TypeNode InsertTypeRule::computeType(NodeManager* nodeManager,
   return setType;
 }
 
+TypeNode SetMapTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode SetMapTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -332,6 +384,10 @@ TypeNode SetMapTypeRule::computeType(NodeManager* nodeManager,
   return retType;
 }
 
+TypeNode SetFilterTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode SetFilterTypeRule::computeType(NodeManager* nodeManager,
                                         TNode n,
                                         bool check,
@@ -375,6 +431,10 @@ TypeNode SetFilterTypeRule::computeType(NodeManager* nodeManager,
   return setType;
 }
 
+TypeNode SetFoldTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode SetFoldTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
                                       bool check,
@@ -427,6 +487,10 @@ TypeNode SetFoldTypeRule::computeType(NodeManager* nodeManager,
   return retType;
 }
 
+TypeNode RelBinaryOperatorTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode RelBinaryOperatorTypeRule::computeType(NodeManager* nodeManager,
                                                 TNode n,
                                                 bool check,
@@ -486,6 +550,10 @@ TypeNode RelBinaryOperatorTypeRule::computeType(NodeManager* nodeManager,
   return resultType;
 }
 
+TypeNode RelTransposeTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode RelTransposeTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
                                            bool check,
@@ -503,6 +571,10 @@ TypeNode RelTransposeTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->mkSetType(nodeManager->mkTupleType(tupleTypes));
 }
 
+TypeNode RelTransClosureTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode RelTransClosureTypeRule::computeType(NodeManager* nodeManager,
                                               TNode n,
                                               bool check,
@@ -533,6 +605,10 @@ TypeNode RelTransClosureTypeRule::computeType(NodeManager* nodeManager,
   return setType;
 }
 
+TypeNode JoinImageTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode JoinImageTypeRule::computeType(NodeManager* nodeManager,
                                         TNode n,
                                         bool check,
@@ -577,6 +653,10 @@ TypeNode JoinImageTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->mkSetType(nodeManager->mkTupleType(newTupleTypes));
 }
 
+TypeNode RelIdenTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode RelIdenTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
                                       bool check,
@@ -602,6 +682,10 @@ TypeNode RelIdenTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->mkSetType(nodeManager->mkTupleType(tupleTypes));
 }
 
+TypeNode RelationGroupTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode RelationGroupTypeRule::computeType(NodeManager* nm,
                                             TNode n,
                                             bool check,
@@ -638,6 +722,10 @@ TypeNode RelationGroupTypeRule::computeType(NodeManager* nm,
   return nm->mkSetType(setType);
 }
 
+TypeNode RelationAggregateTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode RelationAggregateTypeRule::computeType(NodeManager* nm,
                                                 TNode n,
                                                 bool check,
@@ -705,6 +793,10 @@ TypeNode RelationAggregateTypeRule::computeType(NodeManager* nm,
   return nm->mkSetType(functionType.getRangeType());
 }
 
+TypeNode RelationProjectTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode RelationProjectTypeRule::computeType(NodeManager* nm,
                                               TNode n,
                                               bool check,
