@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Andrew Reynolds, Andres Noetzli, Gereon Kremer
+ *   Andrew Reynolds, Andres Noetzli, Tianyi Liang
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -449,73 +449,99 @@ Node CoreSolver::checkCycles( Node eqc, std::vector< Node >& curr, std::vector<
     //look at all terms in this equivalence class
     eq::EqualityEngine* ee = d_state.getEqualityEngine();
     eq::EqClassIterator eqc_i = eq::EqClassIterator( eqc, ee );
+    const std::set<Node>& rlvSet = d_termReg.getRelevantTermSet();
+    bool hasRlv = false;
     while( !eqc_i.isFinished() ) {
       Node n = (*eqc_i);
-      if( !d_bsolver.isCongruent(n) ){
-        if( n.getKind() == kind::STRING_CONCAT ){
-          Trace("strings-cycle") << eqc << " check term : " << n << " in " << eqc << std::endl;
-          if (eqc != emp)
+      ++eqc_i;
+      if (rlvSet.find(n) == rlvSet.end())
+      {
+        // not relevant, skip
+        continue;
+      }
+      hasRlv = true;
+      if (n.getKind() != kind::STRING_CONCAT || d_bsolver.isCongruent(n))
+      {
+        continue;
+      }
+      Trace("strings-cycle")
+          << eqc << " check term : " << n << " in " << eqc << std::endl;
+      if (eqc != emp)
+      {
+        d_eqc[eqc].push_back(n);
+      }
+      size_t nchild = n.getNumChildren();
+      for (size_t i=0; i<nchild; i++)
+      {
+        Node nc = n[i];
+        Node nr = d_state.getRepresentative(nc);
+        if (eqc == emp)
+        {
+          // for empty eqc, ensure all components are empty
+          if (nr != emp)
           {
-            d_eqc[eqc].push_back( n );
+            std::vector<Node> exps;
+            exps.push_back(n.eqNode(emp));
+            d_im.sendInference(
+                exps, nc.eqNode(emp), InferenceId::STRINGS_I_CYCLE_E);
+            return Node::null();
           }
-          for( unsigned i=0; i<n.getNumChildren(); i++ ){
-            Node nr = d_state.getRepresentative(n[i]);
-            if (eqc == emp)
+        }
+        else
+        {
+          if (nr != emp)
+          {
+            d_flat_form[n].push_back(nr);
+            d_flat_form_index[n].push_back(i);
+          }
+          // for non-empty eqc, recurse and see if we find a loop
+          Node ncy = checkCycles(nr, curr, exp);
+          if (!ncy.isNull())
+          {
+            Trace("strings-cycle") << eqc << " cycle: " << ncy << " at " << n
+                                   << "[" << i << "] : " << nc << std::endl;
+            d_im.addToExplanation(n, eqc, exp);
+            d_im.addToExplanation(nr, nc, exp);
+            if (ncy == eqc)
             {
-              //for empty eqc, ensure all components are empty
-              if (nr != emp)
+              // can infer all other components must be empty
+              for (size_t j = 0; j < nchild; j++)
               {
-                std::vector<Node> exps;
-                exps.push_back(n.eqNode(emp));
-                d_im.sendInference(
-                    exps, n[i].eqNode(emp), InferenceId::STRINGS_I_CYCLE_E);
-                return Node::null();
-              }
-            }else{
-              if (nr != emp)
-              {
-                d_flat_form[n].push_back( nr );
-                d_flat_form_index[n].push_back( i );
-              }
-              //for non-empty eqc, recurse and see if we find a loop
-              Node ncy = checkCycles( nr, curr, exp );
-              if( !ncy.isNull() ){
-                Trace("strings-cycle") << eqc << " cycle: " << ncy << " at " << n << "[" << i << "] : " << n[i] << std::endl;
-                d_im.addToExplanation(n, eqc, exp);
-                d_im.addToExplanation(nr, n[i], exp);
-                if( ncy==eqc ){
-                  //can infer all other components must be empty
-                  for( unsigned j=0; j<n.getNumChildren(); j++ ){
-                    //take first non-empty
-                    if (j != i && !d_state.areEqual(n[j], emp))
-                    {
-                      d_im.sendInference(
-                          exp, n[j].eqNode(emp), InferenceId::STRINGS_I_CYCLE);
-                      return Node::null();
-                    }
-                  }
-                  Trace("strings-error") << "Looping term should be congruent : " << n << " " << eqc << " " << ncy << std::endl;
-                  //should find a non-empty component, otherwise would have been singular congruent (I_Norm_S)
-                  Assert(false);
-                }else{
-                  return ncy;
-                }
-              }else{
-                if (d_im.hasProcessed())
+                // take first non-empty
+                if (j != i && !d_state.areEqual(n[j], emp))
                 {
+                  d_im.sendInference(
+                      exp, n[j].eqNode(emp), InferenceId::STRINGS_I_CYCLE);
                   return Node::null();
                 }
               }
+              Trace("strings-error")
+                  << "Looping term should be congruent : " << n << " " << eqc
+                  << " " << ncy << std::endl;
+              // should find a non-empty component, otherwise would have been
+              // singular congruent (I_Norm_S)
+              Assert(false);
             }
+            else
+            {
+              return ncy;
+            }
+          }
+          else if (d_im.hasProcessed())
+          {
+            return Node::null();
           }
         }
       }
-      ++eqc_i;
     }
     curr.pop_back();
-    Trace("strings-eqc") << "* add string eqc: " << eqc << std::endl;
-    //now we can add it to the list of equivalence classes
-    d_strings_eqc.push_back( eqc );
+    // if there was at least one relevant term, we add to the list
+    if (hasRlv)
+    {
+      Trace("strings-eqc") << "* add string eqc: " << eqc << std::endl;
+      // now we can add it to the list of equivalence classes
+      d_strings_eqc.push_back(eqc);
+    }
   }else{
     //already processed
   }
@@ -645,6 +671,8 @@ void CoreSolver::normalizeEquivalenceClass(Node eqc,
   }
 }
 
+const std::vector<Node>& CoreSolver::getRelevantDeq() const { return d_rlvDeq; }
+
 NormalForm& CoreSolver::getNormalForm(Node n)
 {
   std::map<Node, NormalForm>::iterator itn = d_normal_form.find(n);
@@ -833,9 +861,13 @@ void CoreSolver::getNormalForms(Node eqc,
   Trace("strings-process-debug") << "Get normal forms " << eqc << std::endl;
   eq::EqualityEngine* ee = d_state.getEqualityEngine();
   eq::EqClassIterator eqc_i = eq::EqClassIterator( eqc, ee );
+  const std::set<Node>& rlvSet = d_termReg.getRelevantTermSet();
   while( !eqc_i.isFinished() ){
     Node n = (*eqc_i);
-    if( !d_bsolver.isCongruent(n) ){
+    // this check should be in sync with the check in checkCycles to ensure
+    // we don't compute normal forms for irrelevant terms.
+    if (!d_bsolver.isCongruent(n) && rlvSet.find(n)!=rlvSet.end())
+    {
       Kind nk = n.getKind();
       bool isCLike = utils::isConstantLike(n);
       if (isCLike || nk == STRING_CONCAT)
@@ -2492,7 +2524,7 @@ void CoreSolver::checkNormalFormsDeq()
   const context::CDList<Node>& deqs = d_state.getDisequalityList();
 
   Trace("str-deq") << "Process disequalites..." << std::endl;
-  std::vector<Node> relevantDeqs;
+  d_rlvDeq.clear();
   // for each pair of disequal strings, must determine whether their lengths
   // are equal or disequal
   for (const Node& eq : deqs)
@@ -2513,7 +2545,7 @@ void CoreSolver::checkNormalFormsDeq()
       if (d_state.areEqual(lt[0], lt[1]))
       {
         // if they have equal lengths, we must process the disequality below
-        relevantDeqs.push_back(eq);
+        d_rlvDeq.push_back(eq);
         Trace("str-deq") << "...relevant, lengths equal (" << lt[0] << " "
                          << lt[1] << ")" << std::endl;
       }
@@ -2538,14 +2570,14 @@ void CoreSolver::checkNormalFormsDeq()
     // added splitting lemma above
     return;
   }
-  for (const Node& eq : relevantDeqs)
+  for (const Node& eq : d_rlvDeq)
   {
     Assert(!d_state.isInConflict());
-    // If using the sequence update solver, we always apply extensionality.
+    // If using sequences, we always apply extensionality.
     // This is required for model soundness currently, although we could
     // investigate determining cases where the disequality is already
     // satisfied (for optimization).
-    if (options().strings.stringsDeqExt
+    if (eq[0].getType().isSequence() || options().strings.stringsDeqExt
         || options().strings.seqArray != options::SeqArrayMode::NONE)
     {
       processDeqExtensionality(eq[0], eq[1]);
