@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -79,7 +79,7 @@ TheoryStrings::TheoryStrings(Env& env, OutputChannel& out, Valuation valuation)
                 d_csolver,
                 d_extTheory,
                 d_statistics),
-      d_psolver(env, d_state, d_im, d_termReg, d_bsolver),
+      d_psolver(env, d_state, d_im, d_termReg, d_bsolver, d_csolver),
       d_asolver(env,
                 d_state,
                 d_im,
@@ -288,7 +288,6 @@ struct SortSeqIndex
   {
     Assert(i.first.isConst() && i.first.getType().isInteger()
            && j.first.isConst() && j.first.getType().isInteger());
-    Assert(i.first != j.first);
     return i.first.getConst<Rational>() < j.first.getConst<Rational>();
   }
 };
@@ -320,6 +319,7 @@ bool TheoryStrings::collectModelInfoType(
   std::vector<Node> lts;
   const std::vector<Node> repVec(repSet.at(tn).begin(), repSet.at(tn).end());
   mc->separateByLength(repVec, col, lts);
+  Assert(col.size() == lts.size());
   // indices in col that have lengths that are too big to represent
   std::unordered_set<size_t> oobIndices;
 
@@ -327,8 +327,13 @@ bool TheoryStrings::collectModelInfoType(
   std::map< Node, Node > processed;
   //step 1 : get all values for known lengths
   std::vector< Node > lts_values;
-  std::map<std::size_t, Node> values_used;
-  std::vector<Node> len_splits;
+  // mapping from lengths used to the index in col that used that length
+  std::map<size_t, size_t> values_used;
+  // A list of pairs of indices in col that used the same length term. We use
+  // this as candidates to add length splitting on below (STRINGS_CMI_SPLIT),
+  // which is used as a safeguard when model construction fails unexpectedly
+  // by running out of values.
+  std::vector<std::pair<size_t, size_t>> len_splits;
   for (size_t i = 0, csize = col.size(); i < csize; i++)
   {
     Trace("strings-model") << "Checking length for { " << col[i];
@@ -356,11 +361,11 @@ bool TheoryStrings::collectModelInfoType(
       auto itvu = values_used.find(lvalue);
       if (itvu == values_used.end())
       {
-        values_used[lvalue] = lts[i];
+        values_used[lvalue] = i;
       }
       else
       {
-        len_splits.push_back(lts[i].eqNode(itvu->second));
+        len_splits.emplace_back(i, itvu->second);
       }
       lts_values.push_back(len_value);
     }
@@ -462,7 +467,7 @@ bool TheoryStrings::collectModelInfoType(
         Trace("strings-model")
             << "*** Decide to make length of " << lvalue << std::endl;
         lenValue = nm->mkConstInt(Rational(lvalue));
-        values_used[lvalue] = Node::null();
+        values_used[lvalue] = i;
       }
       // is it an equivalence class with a seq.unit term?
       Node assignedValue;
@@ -674,9 +679,12 @@ bool TheoryStrings::collectModelInfoType(
               // integer equivalence classes that are assigned to the same value
               // in the model.
               AlwaysAssert(!len_splits.empty());
-              for (const Node& sl : len_splits)
+              for (const std::pair<size_t, size_t>& sl : len_splits)
               {
-                Node spl = nm->mkNode(OR, sl, sl.negate());
+                Node s1 = nm->mkNode(STRING_LENGTH, col[sl.first][0]);
+                Node s2 = nm->mkNode(STRING_LENGTH, col[sl.second][0]);
+                Node eq = s1.eqNode(s2);
+                Node spl = nm->mkNode(OR, eq, eq.negate());
                 d_im.lemma(spl, InferenceId::STRINGS_CMI_SPLIT);
                 Trace("strings-lemma")
                     << "Strings::CollectModelInfoSplit: " << spl << std::endl;
@@ -794,7 +802,7 @@ Node TheoryStrings::mkSkeletonFor(Node c)
     Assert(snv.getType() == etn);
     Node v = bvm->mkBoundVar<SeqModelVarAttribute>(snv, etn);
     // use a skolem, not a bound variable
-    Node kv = sm->mkPurifySkolem(v, "smv");
+    Node kv = sm->mkPurifySkolem(v);
     skChildren.push_back(utils::mkUnit(tn, kv));
   }
   return utils::mkConcat(skChildren, c.getType());
@@ -1112,21 +1120,6 @@ TrustNode TheoryStrings::ppRewrite(TNode atom, std::vector<SkolemLemma>& lems)
 {
   Trace("strings-ppr") << "TheoryStrings::ppRewrite " << atom << std::endl;
   Kind ak = atom.getKind();
-  if (ak == EQUAL)
-  {
-    if (atom[0].getType().isRegExp())
-    {
-      std::stringstream ss;
-      ss << "Equality between regular expressions is not supported";
-      throw LogicException(ss.str());
-    }
-    // always apply aggressive equality rewrites here
-    Node ret = d_rewriter.rewriteEqualityExt(atom);
-    if (ret != atom)
-    {
-      return TrustNode::mkTrustRewrite(atom, ret, nullptr);
-    }
-  }
   if (ak == STRING_FROM_CODE)
   {
     // str.from_code(t) ---> ite(0 <= t < |A|, t = str.to_code(k), k = "")
@@ -1250,6 +1243,27 @@ TrustNode TheoryStrings::ppRewrite(TNode atom, std::vector<SkolemLemma>& lems)
   return ret;
 }
 
+TrustNode TheoryStrings::ppStaticRewrite(TNode atom)
+{
+  Kind ak = atom.getKind();
+  if (ak == EQUAL)
+  {
+    if (atom[0].getType().isRegExp())
+    {
+      std::stringstream ss;
+      ss << "Equality between regular expressions is not supported";
+      throw LogicException(ss.str());
+    }
+    // always apply aggressive equality rewrites here
+    Node ret = d_rewriter.rewriteEqualityExt(atom);
+    if (ret != atom)
+    {
+      return TrustNode::mkTrustRewrite(atom, ret, nullptr);
+    }
+  }
+  return TrustNode::null();
+}
+
 /** run the given inference step */
 void TheoryStrings::runInferStep(InferStep s, Theory::Effort e, int effort)
 {
@@ -1261,29 +1275,45 @@ void TheoryStrings::runInferStep(InferStep s, Theory::Effort e, int effort)
   Trace("strings-process") << "..." << std::endl;
   switch (s)
   {
-    case CHECK_INIT: d_bsolver.checkInit(); break;
-    case CHECK_CONST_EQC: d_bsolver.checkConstantEquivalenceClasses(); break;
-    case CHECK_EXTF_EVAL: d_esolver.checkExtfEval(effort); break;
-    case CHECK_CYCLES: d_csolver.checkCycles(); break;
-    case CHECK_FLAT_FORMS: d_csolver.checkFlatForms(); break;
-    case CHECK_NORMAL_FORMS_EQ_PROP: d_csolver.checkNormalFormsEqProp(); break;
-    case CHECK_NORMAL_FORMS_EQ: d_csolver.checkNormalFormsEq(); break;
-    case CHECK_NORMAL_FORMS_DEQ: d_csolver.checkNormalFormsDeq(); break;
-    case CHECK_CODES: d_psolver.checkCodes(); break;
-    case CHECK_LENGTH_EQC: d_csolver.checkLengthsEqc(); break;
-    case CHECK_SEQUENCES_ARRAY_CONCAT: d_asolver.checkArrayConcat(); break;
-    case CHECK_SEQUENCES_ARRAY: d_asolver.checkArray(); break;
-    case CHECK_SEQUENCES_ARRAY_EAGER: d_asolver.checkArrayEager(); break;
-    case CHECK_REGISTER_TERMS_NF:
+    case InferStep::CHECK_INIT: d_bsolver.checkInit(); break;
+    case InferStep::CHECK_CONST_EQC:
+      d_bsolver.checkConstantEquivalenceClasses();
+      break;
+    case InferStep::CHECK_EXTF_EVAL: d_esolver.checkExtfEval(effort); break;
+    case InferStep::CHECK_CYCLES: d_csolver.checkCycles(); break;
+    case InferStep::CHECK_FLAT_FORMS: d_csolver.checkFlatForms(); break;
+    case InferStep::CHECK_NORMAL_FORMS_EQ_PROP:
+      d_csolver.checkNormalFormsEqProp();
+      break;
+    case InferStep::CHECK_NORMAL_FORMS_EQ:
+      d_csolver.checkNormalFormsEq();
+      break;
+    case InferStep::CHECK_NORMAL_FORMS_DEQ:
+      d_csolver.checkNormalFormsDeq();
+      break;
+    case InferStep::CHECK_CODES: d_psolver.checkCodes(); break;
+    case InferStep::CHECK_LENGTH_EQC: d_csolver.checkLengthsEqc(); break;
+    case InferStep::CHECK_SEQUENCES_ARRAY_CONCAT:
+      d_asolver.checkArrayConcat();
+      break;
+    case InferStep::CHECK_SEQUENCES_ARRAY: d_asolver.checkArray(); break;
+    case InferStep::CHECK_SEQUENCES_ARRAY_EAGER:
+      d_asolver.checkArrayEager();
+      break;
+    case InferStep::CHECK_REGISTER_TERMS_NF:
       d_csolver.checkRegisterTermsNormalForms();
       break;
-    case CHECK_EXTF_REDUCTION_EAGER:
+    case InferStep::CHECK_EXTF_REDUCTION_EAGER:
       d_esolver.checkExtfReductionsEager();
       break;
-    case CHECK_EXTF_REDUCTION: d_esolver.checkExtfReductions(e); break;
-    case CHECK_MEMBERSHIP_EAGER: d_rsolver.checkMembershipsEager(); break;
-    case CHECK_MEMBERSHIP: d_rsolver.checkMemberships(e); break;
-    case CHECK_CARDINALITY: d_bsolver.checkCardinality(); break;
+    case InferStep::CHECK_EXTF_REDUCTION:
+      d_esolver.checkExtfReductions(e);
+      break;
+    case InferStep::CHECK_MEMBERSHIP_EAGER:
+      d_rsolver.checkMembershipsEager();
+      break;
+    case InferStep::CHECK_MEMBERSHIP: d_rsolver.checkMemberships(e); break;
+    case InferStep::CHECK_CARDINALITY: d_bsolver.checkCardinality(); break;
     default: Unreachable(); break;
   }
   Trace("strings-process") << "Done " << s
@@ -1304,7 +1334,7 @@ void TheoryStrings::runStrategy(Theory::Effort e)
   {
     InferStep curr = it->first;
     int effort = it->second;
-    if (curr == BREAK)
+    if (curr == InferStep::BREAK)
     {
       // if we have a pending inference or lemma, we will process it
       if (d_im.hasProcessed())
