@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Andrew Reynolds, Aina Niemetz, Yoni Zohar
+ *   Andrew Reynolds, Aina Niemetz, Tianyi Liang
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -25,6 +25,10 @@ namespace cvc5::internal {
 namespace theory {
 namespace strings {
 
+TypeNode StringConcatTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode StringConcatTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
                                            bool check,
@@ -59,6 +63,10 @@ TypeNode StringConcatTypeRule::computeType(NodeManager* nodeManager,
   return tret;
 }
 
+TypeNode StringSubstrTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode StringSubstrTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
                                            bool check,
@@ -88,6 +96,10 @@ TypeNode StringSubstrTypeRule::computeType(NodeManager* nodeManager,
   return t;
 }
 
+TypeNode StringUpdateTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode StringUpdateTypeRule::computeType(NodeManager* nodeManager,
                                            TNode n,
                                            bool check,
@@ -117,6 +129,10 @@ TypeNode StringUpdateTypeRule::computeType(NodeManager* nodeManager,
   return t;
 }
 
+TypeNode StringAtTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode StringAtTypeRule::computeType(NodeManager* nodeManager,
                                        TNode n,
                                        bool check,
@@ -140,6 +156,10 @@ TypeNode StringAtTypeRule::computeType(NodeManager* nodeManager,
   return t;
 }
 
+TypeNode StringIndexOfTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->integerType();
+}
 TypeNode StringIndexOfTypeRule::computeType(NodeManager* nodeManager,
                                             TNode n,
                                             bool check,
@@ -171,6 +191,10 @@ TypeNode StringIndexOfTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->integerType();
 }
 
+TypeNode StringReplaceTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode StringReplaceTypeRule::computeType(NodeManager* nodeManager,
                                             TNode n,
                                             bool check,
@@ -204,6 +228,10 @@ TypeNode StringReplaceTypeRule::computeType(NodeManager* nodeManager,
   return t;
 }
 
+TypeNode StringStrToBoolTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode StringStrToBoolTypeRule::computeType(NodeManager* nodeManager,
                                               TNode n,
                                               bool check,
@@ -222,6 +250,10 @@ TypeNode StringStrToBoolTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode StringStrToIntTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->integerType();
+}
 TypeNode StringStrToIntTypeRule::computeType(NodeManager* nodeManager,
                                              TNode n,
                                              bool check,
@@ -240,6 +272,10 @@ TypeNode StringStrToIntTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->integerType();
 }
 
+TypeNode StringStrToStrTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode StringStrToStrTypeRule::computeType(NodeManager* nodeManager,
                                              TNode n,
                                              bool check,
@@ -258,6 +294,10 @@ TypeNode StringStrToStrTypeRule::computeType(NodeManager* nodeManager,
   return t;
 }
 
+TypeNode StringRelationTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->booleanType();
+}
 TypeNode StringRelationTypeRule::computeType(NodeManager* nodeManager,
                                              TNode n,
                                              bool check,
@@ -281,6 +321,10 @@ TypeNode StringRelationTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->booleanType();
 }
 
+TypeNode RegExpRangeTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->regExpType();
+}
 TypeNode RegExpRangeTypeRule::computeType(NodeManager* nodeManager,
                                           TNode n,
                                           bool check,
@@ -303,6 +347,10 @@ TypeNode RegExpRangeTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->regExpType();
 }
 
+TypeNode StringToRegExpTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return nm->regExpType();
+}
 TypeNode StringToRegExpTypeRule::computeType(NodeManager* nodeManager,
                                              TNode n,
                                              bool check,
@@ -325,6 +373,10 @@ bool StringToRegExpTypeRule::computeIsConst(NodeManager* nodeManager, TNode n)
   return n[0].isConst();
 }
 
+TypeNode ConstSequenceTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode ConstSequenceTypeRule::computeType(NodeManager* nodeManager,
                                             TNode n,
                                             bool check,
@@ -334,6 +386,10 @@ TypeNode ConstSequenceTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->mkSequenceType(n.getConst<Sequence>().getType());
 }
 
+TypeNode SeqUnitTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode SeqUnitTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
                                       bool check,
@@ -344,6 +400,10 @@ TypeNode SeqUnitTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->mkSequenceType(argType);
 }
 
+TypeNode SeqNthTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode SeqNthTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
