@@ -1,10 +1,10 @@
 /******************************************************************************
  * Top contributors (to current version):
- *   Andrew Reynolds, Aina Niemetz
+ *   Andrew Reynolds
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -38,6 +38,8 @@ void TheoryEngineModule::notifyLemma(TNode n,
 {
 }
 
+bool TheoryEngineModule::needsCandidateModel() { return false; }
+
 void TheoryEngineModule::notifyCandidateModel(TheoryModel* m) {}
 
 }  // namespace theory
