@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -29,6 +29,10 @@ namespace cvc5::internal {
 namespace theory {
 namespace uf {
 
+TypeNode UfTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode UfTypeRule::computeType(NodeManager* nodeManager,
                                  TNode n,
                                  bool check,
@@ -71,6 +75,11 @@ TypeNode UfTypeRule::computeType(NodeManager* nodeManager,
   return fType.getRangeType();
 }
 
+TypeNode CardinalityConstraintOpTypeRule::preComputeType(NodeManager* nm,
+                                                         TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode CardinalityConstraintOpTypeRule::computeType(NodeManager* nodeManager,
                                                       TNode n,
                                                       bool check,
@@ -93,14 +102,11 @@ TypeNode CardinalityConstraintOpTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->builtinOperatorType();
 }
 
-TypeNode CardinalityConstraintTypeRule::computeType(NodeManager* nodeManager,
-                                                    TNode n,
-                                                    bool check,
-                                                    std::ostream* errOut)
+TypeNode CombinedCardinalityConstraintOpTypeRule::preComputeType(
+    NodeManager* nm, TNode n)
 {
-  return nodeManager->booleanType();
+  return TypeNode::null();
 }
-
 TypeNode CombinedCardinalityConstraintOpTypeRule::computeType(
     NodeManager* nodeManager, TNode n, bool check, std::ostream* errOut)
 {
@@ -117,12 +123,10 @@ TypeNode CombinedCardinalityConstraintOpTypeRule::computeType(
   return nodeManager->builtinOperatorType();
 }
 
-TypeNode CombinedCardinalityConstraintTypeRule::computeType(
-    NodeManager* nodeManager, TNode n, bool check, std::ostream* errOut)
+TypeNode HoApplyTypeRule::preComputeType(NodeManager* nm, TNode n)
 {
-  return nodeManager->booleanType();
+  return TypeNode::null();
 }
-
 TypeNode HoApplyTypeRule::computeType(NodeManager* nodeManager,
                                       TNode n,
                                       bool check,
@@ -163,6 +167,10 @@ TypeNode HoApplyTypeRule::computeType(NodeManager* nodeManager,
   }
 }
 
+TypeNode LambdaTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode LambdaTypeRule::computeType(NodeManager* nodeManager,
                                      TNode n,
                                      bool check,
@@ -184,6 +192,10 @@ TypeNode LambdaTypeRule::computeType(NodeManager* nodeManager,
   return nodeManager->mkFunctionType(argTypes, rangeType);
 }
 
+TypeNode FunctionArrayConstTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode FunctionArrayConstTypeRule::computeType(NodeManager* nodeManager,
                                                  TNode n,
                                                  bool check,
@@ -233,6 +245,10 @@ Node FunctionProperties::mkGroundTerm(TypeNode type)
   return nm->mkNode(kind::LAMBDA, bvl, ret);
 }
 
+TypeNode IntToBitVectorOpTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  return TypeNode::null();
+}
 TypeNode IntToBitVectorOpTypeRule::computeType(NodeManager* nodeManager,
                                                TNode n,
                                                bool check,
@@ -244,10 +260,18 @@ TypeNode IntToBitVectorOpTypeRule::computeType(NodeManager* nodeManager,
   {
     throw TypeCheckingExceptionPrivate(n, "expecting bit-width > 0");
   }
-  return nodeManager->mkFunctionType(nodeManager->integerType(),
-                                     nodeManager->mkBitVectorType(bvSize));
+  return nodeManager->builtinOperatorType();
 }
 
+TypeNode BitVectorConversionTypeRule::preComputeType(NodeManager* nm, TNode n)
+{
+  if (n.getKind() == kind::BITVECTOR_TO_NAT)
+  {
+    return nm->integerType();
+  }
+  size_t bvSize = n.getOperator().getConst<IntToBitVector>();
+  return nm->mkBitVectorType(bvSize);
+}
 TypeNode BitVectorConversionTypeRule::computeType(NodeManager* nodeManager,
                                                   TNode n,
                                                   bool check,
