@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -26,19 +26,17 @@
  */
 
 #include <cvc5/cvc5.h>
+#include <cvc5/cvc5_parser.h>
 
 #include <cassert>
 #include <iostream>
 #include <string>
 
-#include "parser/api/cpp/command.h"
-#include "parser/parser_antlr.h"
-#include "parser/parser_builder.h"
+#include "parser/parser_exception.h"
 
 using namespace cvc5;
 using namespace cvc5::internal;
 using namespace cvc5::parser;
-using namespace cvc5::internal::language;
 
 int runTest();
 
@@ -82,30 +80,31 @@ std::string parse(std::string instr,
   (declare-fun a () (Array U (Array U U)))\n\
   ";
 
-    cvc5::Solver solver;
-    std::string ilang = "LANG_SMTLIB_V2_6";
+  cvc5::Solver solver;
+  std::string ilang = "LANG_SMTLIB_V2_6";
 
-    solver.setOption("input-language", input_language);
-    solver.setOption("output-language", output_language);
-    SymbolManager symman(&solver);
-    std::unique_ptr<Parser> parser(
-        ParserBuilder(&solver, &symman, false)
-            .withInputLanguage(solver.getOption("input-language"))
-            .build());
-    parser->setInput(
-        Input::newStringInput(ilang, declarations, "internal-buffer"));
-    // we don't need to execute the commands, but we DO need to parse them to
-    // get the declarations
-    while (std::unique_ptr<Command> c = parser->nextCommand())
-    {
-      // invoke the command, which may bind symbols
-      c->invoke(&solver, &symman);
-    }
-  assert(parser->done());  // parser should be done
-  parser->setInput(Input::newStringInput(ilang, instr, "internal-buffer"));
-  cvc5::Term e = parser->nextExpression();
+  solver.setOption("input-language", input_language);
+  solver.setOption("output-language", output_language);
+  SymbolManager symman(&solver);
+  InputParser parser(&solver, &symman);
+  std::stringstream ss;
+  ss << declarations;
+  parser.setStreamInput(ilang, ss, "internal-buffer");
+  // we don't need to execute the commands, but we DO need to parse them to
+  // get the declarations
+  std::stringstream tmp;
+  while (std::unique_ptr<Command> c = parser.nextCommand())
+  {
+    // invoke the command, which may bind symbols
+    c->invoke(&solver, &symman, tmp);
+  }
+  assert(parser.done());  // parser should be done
+  std::stringstream ssi;
+  ssi << instr;
+  parser.setStreamInput(ilang, ss, "internal-buffer");
+  cvc5::Term e = parser.nextExpression();
   std::string s = e.toString();
-  assert(parser->nextExpression().isNull());  // next expr should be null
+  assert(parser.nextExpression().isNull());  // next expr should be null
   return s;
 }
 
@@ -117,13 +116,13 @@ std::string translate(std::string instr,
   assert(output_language == "smt2");
 
   std::cout << "==============================================" << std::endl
-            << "translating from " << Language::LANG_SMTLIB_V2_6 << " to "
-            << Language::LANG_SMTLIB_V2_6 << " this string:" << std::endl
+            << "translating from " << input_language << " to "
+            << output_language << " this string:" << std::endl
             << instr << std::endl;
   std::string outstr = parse(instr, input_language, output_language);
   std::cout << "got this:" << std::endl
             << outstr << std::endl
-            << "reparsing as " << Language::LANG_SMTLIB_V2_6 << std::endl;
+            << "reparsing as " << output_language << std::endl;
   std::string poutstr = parse(outstr, output_language, output_language);
   assert(outstr == poutstr);
   std::cout << "got same expressions " << outstr << " and " << poutstr
@@ -136,7 +135,7 @@ void runTestString(std::string instr, std::string instr_language)
 {
   std::cout << std::endl
             << "starting with: " << instr << std::endl
-            << "   in language " << Language::LANG_SMTLIB_V2_6 << std::endl;
+            << "   in language " << instr_language << std::endl;
   std::string smt2str = translate(instr, instr_language, "smt2");
   std::cout << "in SMT2      : " << smt2str << std::endl;
   std::string outstr = translate(smt2str, "smt2", "smt2");
