@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -31,10 +31,12 @@ class TestApiTypes : public ::testing::Test
 TEST_F(TestApiTypes, printEnum)
 {
   std::stringstream ss;
+  ss << cvc5::SortKind::ARRAY_SORT;
   ss << cvc5::UnknownExplanation::UNKNOWN_REASON;
   ss << cvc5::modes::BlockModelsMode::LITERALS;
-  ss << cvc5::modes::LearnedLitType::LEARNED_LIT_PREPROCESS;
-  ss << cvc5::modes::ProofComponent::PROOF_COMPONENT_FULL;
+  ss << cvc5::modes::LearnedLitType::PREPROCESS;
+  ss << cvc5::modes::ProofComponent::FULL;
+  ss << cvc5::modes::FindSynthTarget::ENUM;
 }
 
 }  // namespace test
