@@ -4,7 +4,7 @@
 #
 # This file is part of the cvc5 project.
 #
-# Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+# Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
 # in the top-level source directory and their institutional affiliations.
 # All rights reserved.  See the file COPYING in the top-level source
 # directory for licensing information.
@@ -15,7 +15,7 @@ import pytest
 import cvc5
 import sys
 
-from cvc5 import Kind, SortKind, BlockModelsMode, RoundingMode, LearnedLitType, ProofComponent
+from cvc5 import Kind, SortKind, BlockModelsMode, RoundingMode, LearnedLitType, ProofComponent, FindSynthTarget
 
 
 @pytest.fixture
@@ -439,6 +439,14 @@ def test_mk_floating_point(solver):
     with pytest.raises(RuntimeError):
         solver.mkFloatingPoint(3, 5, t2)
 
+    sign = solver.mkBitVector(1)
+    exp = solver.mkBitVector(5)
+    sig = solver.mkBitVector(10)
+    bv = solver.mkBitVector(16)
+    a = solver.mkFloatingPoint(
+            sign, exp, sig)
+    assert solver.mkFloatingPoint(
+            sign, exp, sig) == solver.mkFloatingPoint(5, 11, bv)
     slv = cvc5.Solver()
     slv.mkFloatingPoint(3, 5, t1)
 
@@ -902,23 +910,12 @@ def test_mk_true(solver):
 
 
 def test_mk_tuple(solver):
-    solver.mkTuple([solver.mkBitVectorSort(3)],
-                   [solver.mkBitVector(3, "101", 2)])
-    with pytest.raises(RuntimeError):
-      solver.mkTuple([solver.getRealSort()], [solver.mkInteger("5")])
-
-    with pytest.raises(RuntimeError):
-        solver.mkTuple([], [solver.mkBitVector(3, "101", 2)])
-    with pytest.raises(RuntimeError):
-        solver.mkTuple([solver.mkBitVectorSort(4)],
-                       [solver.mkBitVector(3, "101", 2)])
-    with pytest.raises(RuntimeError):
-        solver.mkTuple([solver.getIntegerSort()], [solver.mkReal("5.3")])
+    solver.mkTuple([solver.mkBitVector(3, "101", 2)])
+    solver.mkTuple([solver.mkInteger("5")])
+    solver.mkTuple([solver.mkReal("5.3")])
     slv = cvc5.Solver()
-    slv.mkTuple([solver.mkBitVectorSort(3)],
-                [slv.mkBitVector(3, "101", 2)])
-    slv.mkTuple([slv.mkBitVectorSort(3)],
-                [solver.mkBitVector(3, "101", 2)])
+    slv.mkTuple([slv.mkBitVector(3, "101", 2)])
+    slv.mkTuple([solver.mkBitVector(3, "101", 2)])
 
 
 def test_mk_universe_set(solver):
@@ -947,6 +944,19 @@ def test_mk_const(solver):
     slv = cvc5.Solver()
     slv.mkConst(boolSort)
 
+def test_declare_fun_fresh(solver):
+    boolSort = solver.getBooleanSort()
+    intSort = solver.getIntegerSort()
+    t1 = solver.declareFun("b", [], boolSort, True)
+    t2 = solver.declareFun("b", [], boolSort, False)
+    t3 = solver.declareFun("b", [], boolSort, False)
+    assert t1!=t2
+    assert t1!=t3
+    assert t2==t3
+    t4 = solver.declareFun("c", [], boolSort, False)
+    assert t2!=t4
+    t5 = solver.declareFun("b", [], intSort, False)
+    assert t2!=t5
 
 def test_mk_const_array(solver):
     intSort = solver.getIntegerSort()
@@ -991,6 +1001,17 @@ def test_declare_sort(solver):
     solver.declareSort("s", 2)
     solver.declareSort("", 2)
 
+def test_declare_sort_fresh(solver):
+    t1 = solver.declareSort("b", 0, True)
+    t2 = solver.declareSort("b", 0, False)
+    t3 = solver.declareSort("b", 0, False)
+    assert t1!=t2
+    assert t1!=t3
+    assert t2==t3
+    t4 = solver.declareSort("c", 0, False)
+    assert t2!=t4
+    t5 = solver.declareSort("b", 1, False)
+    assert t2!=t5
 
 def test_define_fun(solver):
     bvSort = solver.mkBitVectorSort(32)
@@ -1446,9 +1467,9 @@ def test_get_unsat_core_and_proof(solver):
     assert solver.checkSat().isUnsat()
 
     unsat_core = solver.getUnsatCore()
-    
+
     solver.getProof()
-    solver.getProof(ProofComponent.PROOF_COMPONENT_SAT)
+    solver.getProof(ProofComponent.SAT)
 
     solver.resetAssertions()
     for t in unsat_core:
@@ -1463,7 +1484,7 @@ def test_learned_literals(solver):
         solver.getLearnedLiterals()
     solver.checkSat()
     solver.getLearnedLiterals()
-    solver.getLearnedLiterals(LearnedLitType.LEARNED_LIT_PREPROCESS)
+    solver.getLearnedLiterals(LearnedLitType.PREPROCESS)
 
 def test_learned_literals2(solver):
     solver.setOption("produce-learned-literals", "true")
@@ -1480,8 +1501,35 @@ def test_learned_literals2(solver):
     solver.assertFormula(f0)
     solver.assertFormula(f1)
     solver.checkSat()
-    solver.getLearnedLiterals(LearnedLitType.LEARNED_LIT_INPUT)
+    solver.getLearnedLiterals(LearnedLitType.INPUT)
 
+def test_get_timeout_core_unsat(solver):
+  solver.setOption("timeout-core-timeout", "100")
+  solver.setOption("produce-unsat-cores", "true")
+  intSort = solver.getIntegerSort()
+  x = solver.mkConst(intSort, "x")
+  tt = solver.mkBoolean(True)
+  hard = solver.mkTerm(Kind.EQUAL,
+                       solver.mkTerm(Kind.MULT, x, x),
+                       solver.mkInteger("501240912901901249014210220059591"))
+  solver.assertFormula(tt)
+  solver.assertFormula(hard)
+  res = solver.getTimeoutCore()
+  assert res[0].isUnknown()
+  assert len(res[1]) == 1
+  assert res[1][0] == hard
+
+def test_get_timeout_core(solver):
+  solver.setOption("produce-unsat-cores", "true")
+  ff = solver.mkBoolean(False)
+  tt = solver.mkBoolean(True)
+  solver.assertFormula(tt)
+  solver.assertFormula(ff)
+  solver.assertFormula(tt)
+  res = solver.getTimeoutCore()
+  assert res[0].isUnsat()
+  assert len(res[1]) == 1
+  assert res[1][0] == ff
 
 def test_get_value1(solver):
     solver.setOption("produce-models", "false")
@@ -1792,7 +1840,10 @@ def test_get_statistics(solver):
     solver.assertFormula(f1)
     solver.checkSat()
     s = solver.getStatistics()
-    assert s['cvc5::TERM'] == {'defaulted': False, 'internal': False, 'value': {'GEQ': 3, 'OR': 1}}
+    assert s['cvc5::TERM'] == {
+            'defaulted': False,
+            'internal': False,
+            'value': {'Kind::GEQ': 3, 'Kind::OR': 1}}
     assert s.get(True, False) != {}
 
 def test_set_info(solver):
@@ -2048,33 +2099,6 @@ def test_mk_sygus_grammar(solver):
     slv.mkGrammar([boolVar2], [intVar])
 
 
-def test_synth_inv(solver):
-    solver.setOption("sygus", "true")
-    boolean = solver.getBooleanSort()
-    integer = solver.getIntegerSort()
-
-    nullTerm = cvc5.Term(solver)
-    x = solver.mkVar(boolean)
-
-    start1 = solver.mkVar(boolean)
-    start2 = solver.mkVar(integer)
-
-    g1 = solver.mkGrammar([x], [start1])
-    g1.addRule(start1, solver.mkBoolean(False))
-
-    g2 = solver.mkGrammar([x], [start2])
-    g2.addRule(start2, solver.mkInteger(0))
-
-    solver.synthInv("", [])
-    solver.synthInv("i1", [x])
-    solver.synthInv("i2", [x], g1)
-
-    with pytest.raises(RuntimeError):
-        solver.synthInv("i3", [nullTerm])
-    with pytest.raises(RuntimeError):
-        solver.synthInv("i4", [x], g2)
-
-
 def test_add_sygus_constraint(solver):
     solver.setOption("sygus", "true")
     nullTerm = cvc5.Term(solver)
@@ -2255,6 +2279,40 @@ def test_check_synth_next3(solver):
     with pytest.raises(RuntimeError):
         solver.checkSynthNext()
 
+def test_find_synth(solver):
+    solver.setOption("sygus", "true")
+    boolSort = solver.getBooleanSort()
+    start = solver.mkVar(boolSort)
+    g = solver.mkGrammar([], [start])
+    truen = solver.mkBoolean(True)
+    falsen = solver.mkBoolean(False)
+    g.addRule(start, truen)
+    g.addRule(start, falsen)
+    f = solver.synthFun("f", [], solver.getBooleanSort(), g)
+
+    # should enumerate based on the grammar of the function to synthesize above
+    t = solver.findSynth(FindSynthTarget.ENUM)
+    assert not t.isNull() and t.getSort().isBoolean()
+
+
+def test_find_synth2(solver):
+    solver.setOption("sygus", "true")
+    solver.setOption("incremental", "true")
+    boolSort = solver.getBooleanSort()
+    start = solver.mkVar(boolSort)
+    g = solver.mkGrammar([], [start])
+    truen = solver.mkBoolean(True)
+    falsen = solver.mkBoolean(False)
+    g.addRule(start, truen)
+    g.addRule(start, falsen)
+
+    # should enumerate true/false
+    t = solver.findSynth(FindSynthTarget.ENUM, g)
+    assert not t.isNull() and t.getSort().isBoolean()
+    t = solver.findSynthNext()
+    assert not t.isNull() and t.getSort().isBoolean()
+
+
 def test_get_abduct(solver):
     solver.setLogic("QF_LIA")
     solver.setOption("produce-abducts", "true")
@@ -2649,7 +2707,6 @@ def test_mk_sygus_var(solver):
     with pytest.raises(RuntimeError):
         solver.declareSygusVar("", cvc5.Sort(solver))
     slv = cvc5.Solver()
-    solver.setOption("sygus", "true")
     with pytest.raises(RuntimeError):
         slv.declareSygusVar("", boolSort)
 
@@ -2691,17 +2748,13 @@ def test_synth_fun(solver):
 
 
 def test_tuple_project(solver):
-    sorts = [solver.getBooleanSort(),\
-                               solver.getIntegerSort(),\
-                               solver.getStringSort(),\
-                               solver.mkSetSort(solver.getStringSort())]
     elements = [\
         solver.mkBoolean(True), \
         solver.mkInteger(3),\
         solver.mkString("C"),\
         solver.mkTerm(Kind.SET_SINGLETON, solver.mkString("Z"))]
 
-    tuple = solver.mkTuple(sorts, elements)
+    tuple = solver.mkTuple(elements)
 
     indices1 = []
     indices2 = [0]
