@@ -4,7 +4,7 @@
  *
  * This file is part of the cvc5 project.
  *
- * Copyright (c) 2009-2022 by the authors listed in the file AUTHORS
+ * Copyright (c) 2009-2023 by the authors listed in the file AUTHORS
  * in the top-level source directory and their institutional affiliations.
  * All rights reserved.  See the file COPYING in the top-level source
  * directory for licensing information.
@@ -128,7 +128,7 @@ class DummyOutputChannel : public theory::OutputChannel
     push(LEMMA, n.getNode());
   }
 
-  void requirePhase(TNode, bool) override {}
+  void preferPhase(TNode, bool) override {}
   void setModelUnsound(theory::IncompleteId id) override {}
   void setRefutationUnsound(theory::IncompleteId id) override {}
 
